"""This module is a Levenberg-Marquardt optimizer with
stem-size control for numeric differentiation.
It just needs a function that produces residuals.
It is multi-threaded, so calculations of residuals can
be farmed out to many processors.
"""

import sys
import threading

if 1:
	import LinearAlgebra as LA
	import RandomArray as RA
	import Numeric as Num
	from Numeric import NewAxis, matrixmultiply
	_floatcodes = (Num.Float, Num.Float0, Num.Float32, Num.Float64)
else:
	import numarray as Num
	import numarray.random_array.RandomArray2      as RA
	import numarray.linear_algebra.LinearAlgebra2  as LA
	from numarray import NewAxis, matrixmultiply
	_floatcodes = (Num.Float, Num.Float32, Num.Float64)

import math
import os
import die
import traceback
import random
import avio
import gpkmisc

class OptError(RuntimeError):
	def __init__(self, s=None):
		RuntimeError.__init__(self, s)

class NoDownhill(OptError):
	def __init__(self, s=None):
		OptError.__init__(self, s)

class NoDerivative(OptError):
	def __init__(self, s=None):
		OptError.__init__(self, s)

class BadParamError(ValueError):
	__doc__ = """Raised when you give and opt instance some invalid control parameter."""

	def __init__(self, s=None):
		ValueError.__init__(self, s)

class BadResult(OptError):
	__doc__ = """The function to be optimized returns some illegal result.
		For instance, an array of the wrong size or type,
		or it returns None for the initial evaluation."""

	def __init__(self, s=None):
		OptError.__init__(self, s)



def vec_equal(a, b):
	"""Check that two Numeric vectors are exactly equal."""
	return Num.alltrue(Num.equal(a, b))


def sumsq(a):
	"""This is the overall error measure."""
	# print "sumsq: a=", a, a.shape
	# return Num.sum(a**2)
	return Num.sum(Num.sort(a**2))


def maxabs(a):
	"""This is used as a measure of how much the function changed as a
	result of a parameter change."""
	absa = Num.absolute(a)
	return absa[Num.argmax(absa)]


def wtf(r_dz, delta, quantum):
	"""This shows how important different measurements are to
	the final derivitive estimate.    The weight must be small
	when the change of a parameter is smaller than the quantum
	(i.e. when r_delta < quantum).  It must also be small when
	the change in z is much larger than T (r_dz>>1).
	It is largest when r_dz is about 1.
	"""
	z_term = abs(r_dz)/(1.0 + r_dz**2)
	if quantum > 0:
		r_delta = delta/quantum
		return (r_delta**2/(1.0 + r_delta**2)) * z_term
	return z_term


def near_duplicate(guess, tmp, quantum):
	for (delt, dz) in tmp:
		if abs(delt-guess) < 0.5*quantum:
			return True
	return False


def symmetry_point(tmp, T, quantum):
	"""Given a list of (delta, z) in a differentiation,
	pick one more point that makes the differentiation more symmetric.
	This is done by picking a new delta that brings
	sum{delta_i * wtf()} closer to zero,
	where wtf() is the weight given to
	each point in the differentiation.
	"""
	sumwd, sumd, sumdzsize, sumwt = (0.0, 0.0, 0.0, 0.0)
	for (delt, dz) in tmp:
		if dz is not None and delt != 0.0:
			dzsize = maxabs(dz)
			wt = wtf(dzsize/T, delt, quantum)
			sumwt += wt
			sumwd += delt*wt
			# print "     (	delt=", delt, "wt=", wt, "dzsize=", dzsize, "sumwd=", sumwd, "dw=", delt*wt, ")"
			sumdzsize += dzsize
			sumd += abs(delt)
	if sumwt <= 0.0:
		return None

	asymmetry = sumwd/sumwt
	dzsize_vs_delt = sumdzsize/sumd
	# print "symmetry_point: asymmetry=", asymmetry, "dzsize_vs_delt=", dzsize_vs_delt
	best = None
	best_asym_after_guess = abs(asymmetry)
	for q in (-0.2, -0.4, -0.5, -0.6, -0.7, -0.8, -0.9, -1.0, -1.1, -1.2, -1.35, -1.5, -1.75, -2, -2.5, -3):
		guess = q * asymmetry
		if near_duplicate(guess, tmp, quantum):
			continue

		wt_guess = wtf(dzsize_vs_delt*guess/T, guess, quantum)
		asym_after_guess = abs(sumwd + guess * wt_guess) / (sumwt + wt_guess)
		# print "	guess=", guess, "sumwd=", sumwd, "wt_guess=", wt_guess, "asym_after_guess=", asym_after_guess
		if asym_after_guess < best_asym_after_guess:
			best_asym_after_guess = asym_after_guess
			best = q*asymmetry
	return best



def scale_est(tmp, T):
	if len(tmp) < 2:
		die.warn("len(tmp)<2 in scale_est")
		return None
	sumd, sumsts = (0.0, 0.0)
	for (delt, dz) in tmp:
		if dz is not None:
			sumsts += maxabs(dz)
			sumd += abs(delt)
	if abs(sumsts) < 1e-10:
		die.warn("sumsts is small (%g/%g) in scale_est" % (sumsts, sumd))
	if sumsts == 0.0:
		return "HUGE"
	s_vs_delt = sumsts/sumd
	return T/s_vs_delt


def explored_region(tmp):
	sumad = 0.0
	for (delt, dz) in tmp:
		sumad += abs(delt)
	return sumad/len(tmp)


def clz_point(qq, T, quantum):
	"""Given a list of (delta, z) tuples in a differentiation attempt,
	find another delta that (a) fills a gap in the sequence,
	(b) is preferably has a sign opposite most of the {delta} values,
	and (c) has a large wtf().
	Points outside the sequence are also considered.
	"""
	if len(qq) == 1:
		return None

	FUDGEFACTOR = 2.0
	tmp = qq[:]
	tmp.sort()
	sumd = 0.0
	for (delt, dz) in tmp:
		if dz is not None:
			wt = wtf(maxabs(dz)/T, delt, quantum)
			sumd += delt*wt		# Find out which side to prefer.

	top = None
	bot = None
	possibilities = []
	for i in range(1, len(tmp)):
		delti, dzi = tmp[i]
		deltim, dzim = tmp[i-1]
		if dzi is None and dzim is None:
			continue

		if dzi is None:
			top = True
			dzc = dzim
		elif dzim is None:
			bot = True
			dzc = dzi
		else:
			dzc = 0.5*(dzi + dzim)

		ctr = 0.5*(delti + deltim)
		if not near_duplicate(ctr, tmp, quantum):
			gap = (delti - deltim)/(abs(delti) + abs(deltim) + quantum)
			wtc = wtf(maxabs(dzc)/T, ctr, quantum)
			gap *= math.sqrt(wtc)
			if ctr * sumd < 0:
				gap *= FUDGEFACTOR
			possibilities.append( (gap, ctr) )

	if not top and len(tmp)>1:
		delti, dzi = tmp[-1]
		assert dzi is not None
		deltim, dzim = tmp[-2]
		ctr = delti + 0.5*(delti-deltim)
		if not near_duplicate(ctr, tmp, quantum):
			gap = (delti - deltim)/(abs(delti) + abs(deltim) + quantum)
			if abs(delti) > abs(deltim):
				dzc = dzi * (ctr/delti)
			else:
				dzc = dzim * (ctr/deltim)
			wtc = wtf(maxabs(dzc)/T, ctr, quantum)
			gap *= math.sqrt(wtc)
			if ctr * sumd < 0:
				gap *= FUDGEFACTOR
			possibilities.append( (gap, ctr) )

	if not bot and len(tmp)>1:
		delti, dzi = tmp[0]
		assert dzi is not None
		deltip, dzip = tmp[1]
		ctr = delti - 0.5*(deltip-delti)
		if not near_duplicate(ctr, tmp, quantum):
			gap = (deltip - delti)/(abs(delti) + abs(deltip) + quantum)
			if abs(delti) > abs(deltip):
				dzc = dzi * (ctr/delti)
			else:
				dzc = dzip * (ctr/deltip)
			wtc = wtf(maxabs(dzc)/T, ctr, quantum)
			gap *= math.sqrt(wtc)
			if ctr * sumd < 0:
				gap *= FUDGEFACTOR
			possibilities.append( (gap, ctr) )

	if len(possibilities)==0:
		return None

	possibilities.sort()
	return possibilities[-1][1]




def deriv_estimate(tmp, T, quantum):
	# This is pretty crude, and could be improved.
	if len(tmp) < 2:
		raise NoDerivative
	sumd = Num.zeros(tmp[0][1].shape, Num.Float)
	sumwt = 0.0
	for (delt, dz) in tmp:
		wt = wtf(maxabs(dz)/T, delt, quantum)
		# print "#WT=", wt, "stepsize=", maxabs(dz), "T=", T, "delt=", delt
		increment = (dz*(delt*wt)).astype(Num.Float)
		Num.add(sumd, increment, sumd)
		sumwt += delt*delt*wt
	assert sumwt > 0, "Whoops!"
	#sumd.savespace(1)
	return sumd/sumwt


def need_more_diff_pts(tmp, T, quantum):
	n = 0
	nT = 0
	nq = 0
	nTl = 0
	for (delt, dz) in tmp:
		if dz is not None:
			n += 1
			if 0.5*T < maxabs(dz) < 2*T:
				nT += 1
			if 0.2*T < maxabs(dz) < 5*T:
				nTl += 1
			if abs(delt) > 0.9*quantum:
				nq += 1
	return n<3 or nT<1 or nq<2 or nTl<2



def diff_one(x, i, sem):
	"""Differentiate z with respect to parameter i.
	We enter this function with the semaphore 'sem' already acquired.
	"""
	die.note("differentiating", i)
	# print "# DIFF", i
	z = x.z()
	zraw = x.p.zraw()
	assert z is not None
	# print "at p=", x.p, "z=", z
	delta = x.scale[i]
	# print "delta=", delta
	v = Num.zeros((x.np,), Num.Float)
	v[i] = 1.0
	# print "v=", v
	tmp = [ (0.0, Num.zeros(z.shape, Num.Float)) ]
	# print "tmp=", tmp
	quantum = x.deriv_quantum[i]

	non_None = 0
	for sign in (-1, 1, -0.5, 0.5, -0.25, 0.25, -0.125, 0.125, -0.0625, 0.0625):
		delta = sign*max(x.scale[i], quantum)
		if near_duplicate(delta, tmp, quantum):
			continue
		# print "# scale=", x.scale[i], "quantum=", quantum, "delta=", delta, "sign=", sign
		q = x.constrain(x.p.p, x.p.p + delta*v, x.args)
		if q is None or vec_equal(q, x.p.p):
			die.warn("Constraint complains: delta=%g" % delta)
			continue
		z1 = prms(q, x.fn, x.args, (sem.id, x.steps), diffparam=i, diffctr=zraw).z()
		if z1 is not None:
			if z1.shape != z.shape:
				sem.release()
				raise BadResult, 'Size mismatch: %s vs. %s' % (str(z1.shape),
											str(z.shape))
			try:
				delta_z = z1 - z
				non_None += 1
			except OverflowError:
				die.warn("Overflow in differentiation attempt")
				delta_z = None

		tmp.append((delta, delta_z))
		
		if non_None > 0:
			break

	if len(tmp) < 2:
		sem.release()
		raise NoDerivative, 'len(tmp)<2, initial'

	parity = True
	while need_more_diff_pts(tmp, x.Td(), quantum):
		if parity:
			delta = clz_point(tmp, x.Td(), quantum)
			if delta is None:
				delta = symmetry_point(tmp, x.Td(), quantum)
			else:
				parity = not parity
		else:
			delta = symmetry_point(tmp, x.Td(), quantum)
			if delta is None:
				delta = clz_point(tmp, x.Td(), quantum)
			else:
				parity = not parity
		if delta is None:
			die.warn("Neither symmetry_point nor clz_point")
			break
		q = x.constrain(x.p.p, x.p.p + delta*v, x.args)
		if q is not None and not vec_equal(q, x.p.p):
			z1 = prms(q, x.fn, x.args, (sem.id, x.steps), diffparam=i, diffctr=zraw).z()
			tmp.append((delta, z1-z))
		else:
			die.warn("Constrain complains 2")

	# We calculate a derivative from the tmp array.
	x.diff[i,:] = deriv_estimate(tmp, x.Td(), quantum)

	# Re-estimate scale:
	newscale = scale_est(tmp, x.Td())
	if newscale == "HUGE":
		die.info("scale_est is huge on %d %g %s" % (i, x.scale[i], str(tmp)))
		x.newscale[i] = 3.0*x.scale[i]
	elif newscale is not None:
		x.newscale[i] = gpkmisc.limit(x.scale[i]*0.1, gpkmisc.geo_mean(x.scale[i], newscale),
					x.scale[i]*10.0)
	else:
		die.info("scale_est is None on %d %g %s" % (i, x.scale[i], str(tmp)))
		x.newscale[i] = x.scale[i]
	x.newscale[i] = max(x.newscale[i], quantum)

	# Used for termination conditions.
	x.explored[i] = explored_region(tmp)
	# print "# newscale=", x.newscale[i], "quantum=", quantum, "explored=", x.explored[i]
	sys.stderr.flush()
	sys.stdout.flush()
	sem.release()



def eval_lambda(processor, a, b, startp, lamb, out, opt):
	rmat = RA.standard_normal(a.shape)
	# The factor of self.np sets the geometric mean of the eigenvalues to 1,
	# so that a typical eigenvalue is close to the eigenvalues of
	# the identity matrix.
	rsq = matrixmultiply(rmat, Num.transpose(rmat)) / a.shape[0]
	aa = a + rsq * lamb

	try:
		s = LA.solve_linear_equations(aa, b) * opt.scale
	except LA.LinAlgError:	# Singular matrix
		processor.release()
		return
	# print "s=", s
	# print "startp.p=", startp.p
	newp = opt.constrain(startp.p, startp.p - s, opt.args)	# Trim the step.
	if newp is None:
		die.warn("eval_lambda: Step outside of valid region" )
		pnew = None
	else:
		pnew = prms(newp, opt.fn, opt.args, (processor.id, opt.steps))
		pnew.sumsq()	# Force evaluation to fill cache.
	out.append((lamb, pnew, rsq))
	processor.release()




def lamb_correct(newscale, scale):
	assert len(newscale.shape) == 1
	assert newscale.shape == scale.shape
	return math.exp(2*Num.sum(Num.log(newscale/scale))/newscale.shape[0])

# threading._VERBOSE = 1

class mysem:
	__doc__ = """This is a semaphore that is automatically released when
			it is de-allocated.   It also contains a processor ID."""

	def __init__(self, id, sem):
		self.id = id
		self.sem = sem

	def release(self):
		self.sem._release(self.id)
		self.sem = None

	def __del__(self):
		if self.sem:
			self.release()


class semclass:
	__doc__ = """This is a semaphore to control the number of
			simultaneous computations.  It hands out a
			mysem class which is used to release the semaphore."""

	def __init__(self, n):
		self.s = threading.Semaphore(n)
		self.p_list = range(n)
		self.nthreads = n
		# print "SEMCLASS:", self.p_list

	def acquire(self):
		# print "SEMCLASS: acquire_wait"
		self.s.acquire()
		tmp = self.p_list.pop(-1)
		# print "SEMCLASS: acquired", tmp, "remaining:", self.p_list
		return mysem(tmp, self)

	def _release(self, id):
		self.p_list.append(id)
		self.s.release()



_NumType = type(Num.array([1], Num.Float))
_tupletype = type(())
_sequenceType = type([])
_floatType = type(1.0)

def _zarray(z):
	if type(z) == _NumType:
		assert z.typecode() in _floatcodes, "Opt needs floating point data (level2), not %s" % z.typecode()
		return Num.ravel(z)
	assert type(z) == _floatType, "Opt needs an array of mixed floats and Num arrays"
	return Num.array([z], Num.Float)


class prms:
	__doc__ = """This class records parameters and caches function evaluations.
		All function evaluations happen here."""

	def __init__(self, p, fn, args, processor, diffparam=None, diffctr=None):
		assert len(p.shape) == 1
		self.p = p
		self.lock = threading.RLock()
		self.fn = fn
		self.args = args
		self.processor = processor
		self.diffparam = diffparam
		self.diffctr = diffctr


	def zraw(self):
		self.lock.acquire()
		if not hasattr(self, 'zr_cache'):
			try:
				self.zr_cache = (self.fn)(self.p, self.args, self.diffparam,
								self.diffctr, self.processor)
			except:
				traceback.print_exc(file=sys.stdout)
				die.die("Uncaught exception in self.fn")
		self.lock.release()
		return self.zr_cache


	def z(self):
		self.lock.acquire()
		if not hasattr(self, 'z_cache'):
			z = self.zraw()
			ztype = type(z)
			if z is None:
				self.z_cache = None
			elif ztype == _NumType:
				assert z.typecode() in _floatcodes, "Opt needs floating point data, not %s" % z.typecode()
				self.z_cache = Num.ravel(z)
			elif ztype == _sequenceType or ztype == _tupletype:	# A list of floats mixed with Num arrays
				self.z_cache = Num.concatenate(map(_zarray, z))
			else:
				raise BadResult, "Unknown type(%s) returned from function" % repr(type(z))
		self.lock.release()
		return self.z_cache


	def sumsq(self):
		self.lock.acquire()
		if not hasattr(self, 'sumsq_cache'):
			z = self.z()
			if z is None:
				self.sumsq_cache = None
			else:
				self.sumsq_cache = sumsq(z)
		self.lock.release()
		return self.sumsq_cache

	def __len__(self):
		return self.p.shape[0]

	def __str__(self):
		return str(self.p)

	def __repr__(self):
		return repr(self.p)


class LockedList:
	def __init__(self, start=None):
		if start is None:
			self.x = []
		else:
			self.x = start
		self.l = threading.Lock()

	def append(self, x):
		self.l.acquire()
		self.x.append(x)
		self.l.release()

	def __len__(self):
		return len(self.x)

	def pop(self, i = -1):
		self.l.acquire()
		try:
			tmp = self.x.pop(i)
		except:
			self.l.release()
			raise
		self.l.release()
		return tmp

	def items(self):
		self.l.acquire()
		tmp = self.x
		self.l.release()
		return tmp


def _dE(T):
	return random.expovariate(1.0) * T


def anneal_guts(p, rv, T, lamb, sem, opt):
	ossq = p.sumsq()
	if ossq is None:
		die.warn("anneal_guts: ossq=None")
		sem.release()
		return

	np = len(p)
	# eval, evec = lapack_dsyevd.dsyevd(opt.curv)
	eval, evec = LA.Heigenvectors(opt.curv)
	r = RA.standard_normal((np,)) * math.sqrt(2*T/np)
	tr = Num.sum(eval)
	EPS = 1e-6
	e = 1.0/Num.sqrt(eval + lamb)
	s = matrixmultiply(r*e, evec) * opt.curv_scale * opt.active
	q = opt.constrain(p.p, p.p + s, opt.args)
	if q is None:
		die.warn("anneal_guts: started outside constraint region")
		sem.release()
		return
	p = prms(q, opt.fn, opt.args, (sem.id, opt.steps))
	ssq = p.sumsq()
	if ssq is None:
		die.warn("anneal_guts: ssq=None")
		sem.release()
		return
	prediction = sumsq(r)
	if opt.verbose:
		print "AGUTS: ssq-ossq=", ssq-ossq, "quad_predict=", prediction, "lambda=", lamb
	if ssq < ossq + 2 * _dE(T):
		rv.append( (p, lamb) )		# Return the result here.
	sys.stdout.flush()
	sem.release()


def _evec_write(log_fd, a_vec, a_val, note):
	Nlim = 500
	o = ['note=%s' % note,  'eigenvalue=%g' % a_val]
	# XXX Below error checks shouldn't be needed...
	try:
		sq = Num.absolute(a_vec)**2
	except OverflowError:
		die.info("note=" + note)
		die.info("a=" + str(a_vec))
		raise
	idx = Num.argsort(sq)
	total = Num.sum(sq)
	ssq = 0.0
	n = 0
	for i in range(idx.shape[0]-1, -1, -1):
		j = idx[i]
		o.append( 'c%d=%.3f' % (j, a_vec[j]))
		ssq += sq[j]
		n += 1
		if ssq > total*(1-n/Nlim):
			break
	log_fd.write('#Evec: ' + '; '.join(o) + '\n')



def _opt_eval(maybe_fcn, arg):
	if callable(maybe_fcn):
		return maybe_fcn(arg)
	return float(maybe_fcn)



class opt:
	__doc__ = """A class that implements a optimizer.  """

	def __init__(self, p, T, fcn, args=None):
		"""Create an optimizer object.
		p = initial parameter vector.
		T = temperature for simulated annealing and numeric differentiation.
		fcn = fcn(p, args, diffparam, diffctr, processor), where
			p = parameters, args = arbitrary reference passed to function,
				diffparam = None (unless differentiating),
				diffctr = None (unless differentiating),
				processor = (proc_id, i_steps),
					where proc_id is in range(self.nthreads)
					and specifies which processor ought
					to be assigned to the task, and
					i_steps is the number of steps the
					optimization has gone through.
			The function is assumed to return either a Num Python array
			or a sequence of arrays and floats, mixed.
		You can also set the following variables between calls to self.step():
		self.scale = estimate of the step size to use in differentiation.
		self.deriv_quantum = minimum step size used in differentiation.
		self.T = simulated annealing temperature.
		self.active = vector of which parameters to optimize.  Only the active
				parameters change their value.
		self.sem = a semaphore to control the number of threads to use
			(if you want to set the # of threads do self.sem = semclass(nnn)).
		self.constrain = a function that constrains the step.
			You can use it to do a constrained fit by having it project
			the proposed step back into the legal volume.
			Called as self.constrain(old_prms, proposed_prms, self.args).
			It returns the constrained step, or None if there isn't any.
		"""
		self.fn = fcn
		self.args = args
		self.steps = 0
		if isinstance(p, prms):
			self.p = p
		else:
			self.p = prms(Num.array(p, Num.Float, copy=True), self.fn, self.args, (0, self.steps))
		self.last_p = self.p	# Maintained by terminate().
		self.np = len(self.p.p)	# Number of parameters
		self.scale = Num.ones((self.np,), Num.Float)
		self.deriv_quantum = None
		self.T = T
		self.active = Num.ones((self.np,), Num.Int)
		try:
			nthreads = os.sysconf('SC_NPROCESSORS_ONLN')
		except (ValueError, AttributeError):
			nthreads = 1
		self.lamb = 1.0
		self.sem = semclass(nthreads)
		self.verbose = 1
		self.constrain = lambda x, y, args: y




	def set_nthreads(self, n):
		"""Set the number of simultaneous threads to be allowed.
		Calculations of the residuals are farmed out to threads.
		"""
		self.sem = semclass(min(self.np, max(0, n)))




	def z(self):
		"""What is the current residual?"""
		return self.p.z()


	def sumsq(self):
		"""What is the current error?"""
		return self.p.sumsq()


	def _differentiate(self, prms_to_measure = None):
		"""Sets self.diff to the Jacobian matrix.  Jacobian is in real units, with
		no parameter scaling.  Also sets self.scale.

		The differentials with respect to unmeasured parameters are left unchanged
		from whatever they were.  This is good under some circumstances, where
		you might be optimizing just a single (of many) local parameter which only
		affects the solution locally, and you have some global parameters which
		you don't expect to change much as the local parameter changes.
		However, sometimes you might want to zero out self.diff to get rid of
		old information.
		"""

		sys.stderr.flush()
		sys.stdout.flush()
		if self.deriv_quantum is None:
			self.deriv_quantum = Num.zeros((self.np,), Num.Float)
		if prms_to_measure is None:
			prms_to_measure = self.active
		zctr = self.z()
		if zctr is None:
			raise BadResult, "Function returns None at starting point."
		no = zctr.shape[0]
		self.explored = Num.ones((self.np,), Num.Float)
		self.newscale = Num.ones((self.np,), Num.Float)
		if not hasattr(self, 'diff') or self.diff is None or self.diff.shape != (self.np, no):
			self.diff = None	# Free the memory.
			self.diff = Num.zeros((self.np, no), Num.Float)
			# self.diff.savespace(1)
		threads = []
		# All the work is done in the threads.
		# This obscure little bit below starts up at most self.nthreads
		# at a time to do self.np differentiations, starting new ones as old
		# ones terminate.
		for i in range(self.np):
			# We start threads to calculate derivatives,
			# making sure not to have
			# more then self.nthreads operating at one time.
			if prms_to_measure[i]:
				processor = self.sem.acquire()
				thr = threading.Thread(target=diff_one,
							args=(self, i, processor))
				threads.append(thr)
				thr.start()

		for thr in threads:
			thr.join()


	def _diff_update(self, oldp, newp):
		"""Adjust the derivitive matrix to match a newly recorded newp.z()
		value.   This is a secant update."""
		if self.verbose:
			print "# diff_update"
		step = newp.p - oldp.p
		step2 = sumsq(step/self.curv_scale)
		if step2 < 1e-10:
			return 0	# Parameters changed by trivial amount.
		# We expect the residuals to change by step*diff

		chg = newp.z() - oldp.z()
		chg_sz = sumsq(chg)

		ex = matrixmultiply(step, self.diff)
		ex_sz = sumsq(ex)
		cor_sz = sumsq(chg - ex)
		# If the residuals don't change much, then reduce the update,
		# because the secant isn't long enough to get a good measurement.
		f = chg_sz/(chg_sz + 0.5*self.Td())
		# We are cautious, and make only a partial update when the secant
		# disagrees badly with our initial estimate.
		f *= (ex_sz / (ex_sz + cor_sz)) * ( chg_sz / (chg_sz + cor_sz))
		if self.verbose:
			print "	f=", f

		if self.verbose:
			print "	change in z=", chg_sz, "expected=", ex_sz, "correction=", cor_sz
		# Correction is a BIG matrix...
		dtc = self.diff.typecode()
		correction = Num.outerproduct(step.astype(dtc), (chg - ex).astype(dtc)) * Num.array(f/step2, dtc)
		Num.add(self.diff, correction, self.diff)
		del correction		# Free some memory...
		self.calc_curv()
		return 1



	def _curv_update(self, oldp, newp):
		"""Adjust the curvature matrix to match a newly recorded newp.sumsq()
		value.   This is a secant update."""
		# expect sumsq to change by 2*resid*diff*s + s*diff*diff*s
		if self.verbose:
			print "# curv_update"
		step = (newp.p - oldp.p)/self.curv_scale
		step2 = sumsq(step)
		EPS = 1e-10
		if step2 < EPS:
			return 0.0

		step_chg = sumsq(newp.z() - oldp.z())
		# If the residuals don't change much, then reduce the update,
		# because the secant isn't long enough to get a good measurement.
		f = step_chg/(step_chg + 10*self.Td())

		b = matrixmultiply( self.diff, oldp.z())
		parab = Num.dot(step, matrixmultiply(self.curv, step))
		if self.verbose:
			print "#	parabolic part:", parab
			print "#	downhill part:", 2*matrixmultiply(step*self.curv_scale, b)
		ex = 2*matrixmultiply(step*self.curv_scale, b) + parab
		if self.verbose:
			print "#	expect a change of", ex, "get", newp.sumsq() - oldp.sumsq()
		delta = (newp.sumsq() - oldp.sumsq()) - ex
		if delta <= 0:
			return 0.0	# Don't let eigenvalues get negative.
			print "#	delta=", delta
		correction = Num.outerproduct(step, step) * (f*delta/step2)
		Num.add(self.curv, correction, self.curv)
		return delta


	LOG_NAME = 'a.tmp'

	def _log(self):
		if not hasattr(self, 'log_fd'):
			self.log_fd = open(self.LOG_NAME, "w")
		return self.log_fd


	def print_ev_diag(self, misc={}, vectors=0):
		log_fd = self._log()
		a = self.curv

		if vectors:
			# eval, evec = lapack_dsyevd.dsyevd(a)
			eval, evec = LA.Heigenvectors(a)
		else:
			# eval, evec = lapack_dsyevd.dsy(a, jobz='N')
			eval = LA.Heigenvalues(a)
			evec = None

		mx_e = gpkmisc.N_maximum(eval)
		o = misc.copy()
		o['Max_EV'] = mx_e
		o['Min_EV'] = gpkmisc.N_minimum(eval)
		lg_thr = mx_e/10.0
		sm_thr = mx_e/1e6
		o['N_large_ev'] = Num.sum(Num.greater(eval, lg_thr))
		o['large_thr'] = lg_thr
		o['N_small_ev'] = Num.sum(Num.less(eval, sm_thr))
		o['small_thr'] = sm_thr
		o['Median_EV'] = gpkmisc.N_median(eval)
		o['N_sm_lambda'] = Num.sum(Num.less(eval, self.lamb))
		o['lambda'] = self.lamb
		log_fd.write('#EV: ' + avio.concoct(o) + '\n')
		if vectors:
			n = eval.shape[0]
			for i in range(n):
				if eval[i] > lg_thr:
					_evec_write(log_fd, evec[i,:], eval[i], "L")
				if eval[i] < sm_thr:
					_evec_write(log_fd, evec[i,:], eval[i], "S")
		log_fd.flush()



	def Td(self):
		"""This allows the differentiation temperature to be dynamically set."""
		if callable(self.T):
			return self.T(self)
		return float(self.T)


	def dE(self):
		return _dE(self.Td())

	def quick_lambda(self, a, b, startp, lamb, ntries=10):
		if self.verbose:
			print "# quick lambda: lamb=", lamb
		attempt = 0
		bestp, bestl, bestrsq = (None, 0.0, None)
		NTRIES = 3 + int(round(math.sqrt(self.np))) * self.sem.nthreads
		EPS = 1e-13
		ev_sum = Num.trace(a)
		lamb = min(lamb/5.0, ev_sum*5.0) * math.exp(-self.sem.nthreads * 0.2)
		rv = LockedList()
		while attempt < NTRIES:
			processor = self.sem.acquire()
			tmp = threading.Thread(target=eval_lambda,
						args=(processor, a, b, startp, lamb, rv, self))
			tmp.start()

			processor = self.sem.acquire()	# A thread may have finished.
			while len(rv) > 0:
				bestl1, bestp1, bestrsq1 = rv.pop(0)
				if bestp1 is None:
					continue
				if bestp1.z() is None:
					continue
				if bestp is None or bestp1.sumsq()<bestp.sumsq():
					bestp, bestl, bestrsq = (bestp1, bestl1, bestrsq1)
				if bestp1.sumsq() < startp.sumsq() + self.dE():
					# Good enough. Don't start any more.
					attempt = NTRIES
				elif sumsq(bestp1.z() - startp.z()) < 0.2 * self.Td():
					# We're too close to the starting point.  Go no closer.
					attempt = NTRIES
			processor.release()
			attempt += 1
			lamb = lamb*2.0*math.pow(10.0, 1.0/self.sem.nthreads) + EPS*ev_sum
		# THIS FUNCTION CAN EXIT WITH CHILD THREADS STILL RUNNING.
		# That's OK; they will eventually terminate and clean themselves up.
		return (bestl, bestp, bestrsq)


	def search_with_update(self, lamb, newp):
		if self.verbose:
			print "search with update: lamb=", lamb, "ssq=", newp.sumsq()
		# self._curv_update(self.p, newp)
		self._diff_update(self.p, newp)
		# self._curv_update(self.p, newp)
		prm_per_proc = float(self.np)/float(self.sem.nthreads)
		tthresh = 2*int(math.ceil(math.sqrt(prm_per_proc/7.0)))
		impthresh = newp.sumsq()/prm_per_proc
		tries = 0
		improvement = 0.0
		bestp = newp
		while tries < tthresh:
			if self.verbose:
				print "	swu: tries=", tries
			# ds = self.scale[:,NewAxis] * self.diff
			# b = matrixmultiply(ds, bestp.z())
			b = self.scale * matrixmultiply(self.diff, bestp.z())
			# Make sure this is the same as above commented out...

			bestl, bestp1, bestrsq1 = self.quick_lambda(self.curv, b, bestp, lamb, 3)
			lamb = max(1e-10, bestl)
			if bestp1 is None:
				if self.verbose:
					print "	None"
				break
			if self.verbose:
				print "	bestp1=", bestp1.sumsq(), "bestp=", bestp.sumsq(), "T=", self.Td()
			if bestp1.sumsq() <= bestp.sumsq():
				self._diff_update(self.p, newp)
				# self._curv_update(bestp, bestp1)
				improvement = bestp.sumsq() - bestp1.sumsq()
				if self.verbose:
					print "	improvement=", improvement, "impthresh=", impthresh
				bestp = bestp1
			else:
				improvement = 0.0

			if improvement < self.Td():
				if self.verbose:
					print "	Not much better"
				break
			elif improvement <= impthresh:
				tries += 1
		return (bestp, bestrsq1)


	def calc_curv(self):
		if self.verbose:
			print "scale=", self.scale
		# DS is a BIG matrix...
		# ds = self.scale[:,NewAxis] * self.diff
		# a = matrixmultiply(ds, Num.transpose(ds))
		a = self.scale[:,NewAxis]	\
			* matrixmultiply(self.diff, Num.transpose(self.diff))	\
			* self.scale[NewAxis, :]
		assert a.shape == (self.np, self.np)
		# Next is kluge to keep inactive parameters from generating numeric problems:
		ev_avg = Num.trace(a)/Num.sum(self.active)
		for i in Num.nonzero(1 - self.active):
			a[i,i] += ev_avg

		# print "a=", a
		self.curv = a	# Use curv_scale with a.
		self.curv_scale = self.scale

	def measure(self):
		"""If you decrease the number of active parameters, it might behoove
		you to set self.diff=None, before you call this function,
		so that old derivitives get flushed.
		"""
		self._differentiate()
		self.calc_curv()
		self.last = 'measure'


	def step(self):
		"""A minimization step."""
		if not self.Td() > 0:
			raise BadParamError, 'Temperature must be positive.'
		if self.verbose:
			print "BEGIN STEP"
		# ds = self.measure()
		# b = matrixmultiply(ds, self.z())
		self.measure()
		b = self.scale * matrixmultiply(self.diff, self.z())
		# print "b=", b

		self.lamb, bestp, bestrsq = self.quick_lambda(self.curv, b, self.p, self.lamb)

		if bestp is None or bestp.sumsq() is None:
			die.warn("No Downhill, quick_lambda. Will anneal.")
			# We use Td instead of Ta below, because this is essentially
			# an attempt to recover from an error, not a simulated
			# anealing step to get error bars.   We want to take a step
			# comparable to the size of the region we've explored during
			# the differentiation, and that region is set by Td.
			self.one_anneal(self.Td())
			return
		if not ( bestp.sumsq() < self.sumsq() ):
			die.warn("No downhill after quick_lambda: sumsq %g -> %g"
						% (self.sumsq(), bestp.sumsq()))

		# Because the time to calculate the derivatives scales
		# as the number of parameters per processor increases, we
		# can do increasingly finer searches in lambda,
		# without dominating the total computational time.
		# bestl1, bestp1 = self.search_lambda(a, b, bestp, self.lamb)
		# if bestp is None or ( bestp1 is not None
					# and bestp1.sumsq()<bestp.sumsq() ):
			# bestp = bestp1
			# self.lamb = bestl1
		bestp1, bestrsq1 = self.search_with_update(self.lamb, bestp)
		if bestp is None or ( bestp1 is not None
					and bestp1.sumsq()<bestp.sumsq() ):
			bestp = bestp1
			bestrsq = bestrsq1


		sys.stderr.flush()
		sys.stdout.flush()
		if bestp.sumsq() < self.sumsq() + self.dE():
			self.p = bestp
			self.lambdir = bestrsq
			self.lamb = max(self.lamb * lamb_correct(self.newscale, self.scale), 1e-10)
			self.scale = self.newscale
			self.steps += 1
			self.last = 'step'
		else:
			die.warn("No Downhill, search. Will anneal.")
			self.one_anneal(self.Td())


	def predicted_sumsq_delta(self, last_p):
		"""The energy change of a step, assuming you're at the minimum."""
		p_step = self.p.p - last_p.p
		ps = p_step / self.curv_scale
		return Num.dot(ps, matrixmultiply(self.curv, ps))




	def one_anneal(self, T):
		"""Take one simulated annealing step, as fast as possible.
		We send all the processors in random directions, and the one
		that reports an acceptable step first is returned.
		Returns 1 on success, 0 if no good step could be found
		in a reasonable number of tries.
		The temperature can also be a function of one argument
		(as an alternative to a float), in which case, the temperature
		is calculated as T(self)."""

		# Allow T to be calculated dynamically.
		Ta = _opt_eval(T, self)

		attempt = 0
		NTRIES = 3 + int(round(math.sqrt(self.np))) * self.sem.nthreads
		EPS = 1e-13
		lamb = self.lamb / 30.0 + EPS*Num.trace(self.curv)
		rv = LockedList()
		self.last = 'anneal'
		while attempt < NTRIES:
			processor = self.sem.acquire()
			tmp = threading.Thread(target=anneal_guts,
						args=(self.p, rv, Ta, lamb, processor, self))
			tmp.start()

			processor = self.sem.acquire()	# A thread may have finished.
			if len(rv) > 0:
				processor.release()
				self.p, rlamb = rv.pop(0)
				self.lamb = math.sqrt(rlamb * self.lamb)
				# THIS FUNCTION CAN EXIT WITH CHILD THREADS STILL RUNNING.
				return 1
			processor.release()
			attempt += 1
			lamb *= math.pow(3.0, 1.0/self.sem.nthreads)
		die.warn("simulated annealing fails after %d attempts" % attempt)
		return 0


	def terminate(self):
		"""Returns number between 0 and 1 if the optimization seems finished.
		Returns -1 if it's clearly not done yet.
		This is designed to be called after step(),
		so that both self.p and self.last_p refer to the results
		of downhill steps.
		"""
		# The step is small compared to the region
		#    that was explored for differentiation:
		# Unfortunately, this test (sort-of-correctly) indicates
		# that optimizations are never done when the chi-squared surface
		# is nearly degenerate.   Thus, we weaken this first test,
		# so termination will eventually happen, but it will be slow
		# if the first condition is not really met.
		p_step = self.p.p - self.last_p.p
		# print "# pstep=", p_step
		# print "# explored=", self.explored
		step_to_explored = Num.sum(Num.absolute(p_step/self.explored))/p_step.shape[0]
		small1 = 1.0/(1 + math.sqrt(step_to_explored))
		if self.verbose:
			print "TERM: step_to_explored=", step_to_explored

		# The real energy change (downward step) is small compared to the temperature:
		delta_E = self.last_p.sumsq() - self.sumsq()
		small2 = delta_E < self.Td()/math.sqrt((self.steps+1)*self.np)
		if self.verbose:
			print "TERM: last_sumsq=", self.last_p.sumsq(),	\
				"sumsq=", self.sumsq(),	\
				"diff=", self.last_p.sumsq() - self.sumsq()

		# The predicted energy change is comparable to the temperature:
		predicted = self.predicted_sumsq_delta(self.last_p)
		small3 = predicted < 10 * self.Td()
		if self.verbose:
			print "TERM: predicted improvement=", predicted, "T=", self.Td()
			print "TERM: tests:", small1, small2, small3

		self.last_p = self.p
		if small2 and small3:
			return small1
		return -1.0


	def covariance_under(self):
		"""This is an underestimate of covariance.
		Adding in the self.lamb term means that it attempts to
		correct for the nonlinearity of the problem,
		at least along the direction of search.
		"""
		c = LA.inverse(self.curv + self.lamb*self.lambdir)
		return self.curv_scale[:,NewAxis] * c * self.curv_scale[NewAxis,:]


	def covariance(self):
		"""Covariance estimate."""
		ident = Num.identity(self.np)
		try:
			c = LA.inverse(self.curv)
		except LA.LinAlgError:
			die.warn("Singular covariance matrix")
			return None
		return self.curv_scale[:,NewAxis] * c * self.curv_scale[NewAxis,:]


	def run(self, maxsteps=None, misc={}, Ta=None):
		"""This is the normal top-level routine.
		You should feel free to make your own, though."""
		if Ta is None:
			Ta = self.Td()
		term = 0.0
		while term < 3 and (maxsteps is None or self.steps<maxsteps):
			self.step()
			self.print_ev_diag(vectors=1)
			ttmp = self.terminate()
			if ttmp < 0:
				term = term/2.0
			else:
				term += self.terminate()
			self.print_at(misc=misc)
			if Ta > 0:
				self.one_anneal(Ta)
				self.print_at(misc=misc)


	def print_errbar(self):
		log_fd = self._log()
		c = self.covariance()
		assert c.shape[0] == c.shape[1]
		log_fd.write('#var:')
		for i in range(c.shape[0]):
			log_fd.write(" %6g" % c[i,i])
		log_fd.write('\n')
		log_fd.flush()


	def sim_anneal(self, nsteps, Ta=None, misc={}):
		"""A top-level simulated annealing routine."""
		if Ta is None:
			Ta = self.Td()
		self.measure()
		self.steps = 0
		no = 1 + self.np/2
		while nsteps is None or self.steps<nsteps:
			if self.steps%self.np == self.np-1 :
				self.measure()
				self.print_errbar()
				self.print_ev_diag(vectors=1)
			if self.steps%no == no-1 :
				self.print_at(misc=misc)
			self.one_anneal(Ta)
			self.steps += 1
		self.print_at(misc=misc)


	def print_at(self, misc={}):
		"""Prints a file to match the old C++ optimizer output."""
		log_fd = self._log()
		log_fd.write("#move:")
		p_step = self.p.p - self.last_p.p
		for i in range(p_step.shape[0]):
			log_fd.write(" %6g" % p_step[i])
		log_fd.write('\n')
		o = misc.copy()
		o['E'] = self.sumsq()
		o['iter'] = self.steps
		o['lambda'] = self.lamb
		o['last'] = self.last
		log_fd.write('# ' + avio.concoct(o) + '\n')
		for i in range(self.p.p.shape[0]):
			log_fd.write(" %g" % self.p.p[i])
		log_fd.write('\n')
		log_fd.flush()
	

def test1_fcn(p, args, *d):
	return p * (1+Num.arrayrange(p.shape[0]))
	

def test1():
	"""Linear."""
	# p = Num.array([1], Num.Float)
	p = Num.array([1, 99, -1, 2, 3, 4, 5, 0, 0, -45, 3, 4, 5, 2, 2, 1, 2], Num.Float)
	o = opt(p, 0.0001, test1_fcn)
	o.set_nthreads( 1 )
	o.scale = Num.ones(p.shape) *  0.1
	o.run()
	print "p=", o.p
	print "cov=", o.covariance()

def test1a_fcn(p, args, *d):
	pp = Num.array([p])
	x = Num.matrixmultiply(pp, args)
	assert x.shape[0] == 1
	return x[0]

def _errsize(a, b):
	q = a - b
	n = Num.sum(Num.ravel(a)**2) + Num.sum(Num.ravel(b)**2)
	s = Num.sum(Num.ravel(q)**2) / (0.5*n)
	return math.sqrt(s)

def test1a():
	"""Linear."""
	# p = Num.array([1], Num.Float)
	p = Num.array([-0.2222, 0.1, 0.5, -1, 5, -0.3], Num.Float)
	q = RA.standard_normal((p.shape[0], p.shape[0]))
	qq = Num.matrixmultiply(q, Num.transpose(q))
	o = opt(p, 0.0001, test1a_fcn, args=q)
	o.scale = Num.exp(RA.standard_normal((p.shape[0],)))
	o.set_nthreads( 1 )
	o.run()
	print "p=", o.p
	print "cov=", o.covariance()
	print "cov_under=", o.covariance_under()
	print "Inv cov=", LA.inverse(o.covariance())
	print "qq=", qq
	print "Inv qq=", LA.inverse(qq)
	print "Inv cov_under=", LA.inverse(o.covariance_under())
	print "cov error=", _errsize(o.covariance(), LA.inverse(qq))
	print "curv error=", _errsize( LA.inverse(o.covariance()), qq)
	ce = o.covariance_under()-LA.inverse(qq)
	print "ce=", ce
	print "cov_under error=", _errsize( o.covariance_under(), LA.inverse(qq))
	print "curv_under error=", _errsize( LA.inverse(o.covariance_under()), qq)

def test2_fcn(p, args, *d):
	return Num.sin(p) - Num.cos(p)


def test2():
	"""Nonlinear"""
	p = Num.array([0, 0.2, 0.4, 0.1, 0.3, 0.5, 0.6, 0.7, 0.8, 0.77, 0.76, 0.33, 0.01], Num.Float)
	o = opt(p, 0.0001, test2_fcn)
	# o.set_nthreads( 1 )
	o.scale = Num.ones(p.shape) *  0.1
	o.run()
	print "p=", o.p


def test3_fcn(p, args, *d):
	o = Num.zeros(p.shape, Num.Float)
	for i in range(p.shape[0]):
		s = 0.0
		for j in range(i):
			s += p[i] - p[j]*(j+1)
		o[i] = s + p[i]
	return o

def test3():
	"""Linear, but correlated"""
	p = Num.array([0, 0.2, 0.4, 0.1, 0.3, 0.5, 0.6, 0.7, 0.8, 0.77, 0.76, 0.33, 0.01], Num.Float)
	o = opt(p, 0.0001, test3_fcn)
	o.scale = Num.ones(p.shape) *  0.1
	o.run()
	print "steps=", o.steps
	print "p=", o.p
	


def test4_fcn(p, args, *d):
	if p[0]<0 or p[1]<0:
		return None
	return [(p[0]-p[1])*100 + p[0], p[1] ]


def test4():
	"""Linear, but with a constraint"""
	p = Num.array([1, 0.1], Num.Float)
	o = opt(p, 0.0001, test4_fcn)
	o.set_nthreads( 1 )
	o.run()
	print "steps=", o.steps
	print "p=", o.p
	

def test5_fcn(p, args, *d):
	o = Num.zeros(p.shape, Num.Float)
	for i in range(p.shape[0]):
		s = 0.0
		for j in range(i):
			s += p[i] - p[j]*(j+1)
		o[i] = s + p[i]
	if gpkmisc.N_maximum(o) > 30  or gpkmisc.N_minimum(o)<-30:
		return None
	return Num.exp(o) - 1


def test5():
	"""Very Nonlinear"""
	p = Num.array([0, 4.2, 1.4, 0.1, 0.8, -1.77, 0.01],
				Num.Float)
	o = opt(p, 0.0001, test5_fcn)
	o.set_nthreads(1)
	o.verbose = 1
	o.scale = Num.ones(p.shape) *  0.1
	try:
		o.run()
	except NoDownhill, q:
		die.warn("No downhill step:" + str(q))
	except:
		raise
	print "p=", o.p


def test6_fcn(p, args, *d):
	o = Num.zeros((3,), Num.Float)
	v = p[0] - p[1]*p[1]
	o[0] = v + 0.001*p[0]
	o[1] = v + 0.001*p[1]
	o[2] = v - 0.001*p[1]
	print "test_fcn: p=", p, "o=", o
	return o



def test6():
	"""Curved valley."""
	p = Num.array([1, 8], Num.Float)
	o = opt(p, 0.001, test6_fcn)
	o.set_nthreads( 1 )
	o.scale = Num.ones(p.shape) *  0.1
	try:
		o.run()
	except NoDownhill, q:
		die.warn("No downhill step:" + str(q))
	except:
		raise
	print "p=", o.p


def _test7_fcn(p, args, *d):
	if p[0]<0 or p[1]<0:
		return None
	return [(p[0]-p[1])*100, p[1]+1 ]


def linconst_min(nparam, param, min):
	"""Generate a linear constraint to be added onto a list and passed
	to linear_constraint().   This constraint expresses that p[param]>=min."""

	vec        = Num.zeros((nparam))
	vec[param] =  1.
	shift      = -1. * min
	return vec, shift


def linconst_max(nparam, param, max):
	"""Generate a linear constraint to be added onto a list and passed
	to linear_constraint().   This constraint expresses that p[param]<=max."""

	vec        = Num.zeros((nparam))
	vec[param] = -1.
	shift      =  1. * max
	return vec, shift



def linear_constraint(op, np, list_of_constraints):
	"""This constrains a step to lie inside a region bounded by a list_of_constraints.
	op = parameters at the beginning of the step.
	np = target parameters.
	list_of_constraints = [ (vector, shift), ... ] where the allowable region of each
		constraint is defined by Num.dot(x, vector)+shift >= 0 .
		Vector is a Num python array, typically.
	"""
	# die.dbg("constraint( %s -> %s )" % (str(op), str(np)))
	dir = np-op
	possibles = list_of_constraints[:]
	while len(possibles)>0:
		# First, we find the nearest constraint that we will encounter.
		eta = None
		whichc = None
		for i in range(len(possibles)):
			cdir, cdist = possibles[i]
			distance = Num.dot(op, cdir) + cdist
			if distance < 0:
				return None	# Whoops. Starting point (old_parameters) fails constraint.
			progress = Num.dot(cdir, dir)
			if progress >= 0:	# Will never hit constraint, assuming we're outside already.
				# die.dbg("Wrong dir to hit constraint %d: %s %g" % (i, str(cdir), cdist))
				continue
			new_eta = -distance / progress
			# print "i=", i, "distance=", distance, "progress=", progress, "new eta=", new_eta
			if new_eta  > 1:	# Won't get there on this step...
				# die.dbg("Constraint %d: %s %g is too far" % (i, str(cdir), cdist))
				continue
			if eta is None or eta > new_eta:
				eta = new_eta
				whichc = i
		if whichc is None:	# We won't hit any more constraints.
			# die.dbg("No more constraints: %d left" % len(possibles))
			return np
		cdir, cdist = possibles.pop(whichc)
		# print "Dir=", dir, "eta=", eta, "np=", np
		np = op + dir*eta	# Move to the intersection.
		# die.dbg("Projecting via %d to %s" % (whichc, str(np)))
		assert abs(Num.dot(np, cdir)+cdist) < 1e-6, "Should go to edge"
		# Future moves will be in the plane of the constraint:
		dir -= cdir*Num.dot(dir, cdir)/Num.dot(cdir, cdir)
	# die.dbg("Used all constraints")
	return np
	

def test7():
	"""Constraint."""
	c = [(Num.array((1, 0), Num.Float), 0), (Num.array((0, 1), Num.Float), 0) ]
	p = Num.array([1, 8], Num.Float)
	o = opt(p, 0.001, _test7_fcn)
	o.set_nthreads( 1 )
	o.constrain = lambda op, np, args, loc=c: linear_constraint(op, np, loc)
	o.run()
	print "p=", o.p




if __name__ == '__main__':
	test1a()




import Num as N
import math


def add_overlap(a, astart, b, bstart):
	"""Add arrays a and b in the overlap region.
	Return (data, start).
	If a, b are time series, they are assumed to have the
	same sampling rate.
	Astart and Bstart apply to the zeroth index.
	All other indices are assumed to match start and length.
	"""
	start = max(astart, bstart)
	end = min(astart+a.shape[0], bstart+b.shape[0])

	out = a[start-astart:end-astart] + b[start-bstart:end-bstart]

	return (out, start)




# def asinh(x):
        # sign = N.sign(x)
	# return sign*N.log(sign*x + N.sqrt(x*x + 1))
asinh = N.arcsinh

# def acosh(x):
	# assert N.all(N.less_equal(x, 1))
	# return N.log(x - N.sqrt(x*x-1))
acosh = N.arccosh


def zero_pad_end(d, padfactor=1):
	if padfactor == 0:
		return N.array(d, copy=1)
	elif padfactor<=0:
		raise ValueError, 'pad factor <= 0'
	assert len(d.shape) == 1
	n = d.shape[0]
	npad = int(round(n*padfactor))
	return N.concatenate( (d, N.zeros((npad,),  N.Float)) )


def Poisson(nbar):
	"""Return a Poisson random integer, whose distribution has
	a mean = nbar.
	"""
	import random
	assert nbar >= 0.0
	if nbar < 20.0:
		L = math.exp(-nbar)
		k = -1
		p = 1.0
		while p >= L:
			k += 1
			p *= random.random()
	else:
		lp = 0.0
		lL = -nbar
		k = 0
		chunk = min( int(round (1 + nbar + 3*math.sqrt(nbar) )), 10000)
		while lp >= lL:
			ptmp = N.log(N.RA.uniform(0.0, 1.0, chunk) )
			lpp = N.add.accumulate(ptmp) + lp
			fsm = N.nonzero(N.less(lpp, lL))[0]
			if fsm.shape[0] != 0:
				k += fsm[0]
				break
			k += ptmp.shape[0]
			lp = lpp[-1]
	return k



def _test_Poisson():
	Nbar = 0.01
	N = int(round(1000/Nbar))
	s = 0
	for i in range(N):
		s += Poisson(Nbar)
	assert abs(s-N*Nbar) < 5*math.sqrt(N*Nbar)


	Nbar = 25.0
	N = 1000
	s = 0
	for i in range(N):
		s += Poisson(Nbar)
	assert abs(s-N*Nbar) < 5*math.sqrt(N*Nbar)



def bevel_concat(a, b, bevel=0, bevel_overlap=1.0, delay=0,
			ta=None, tb=None):
	"""Concatenate two time series.  Bevel the edges,
	and overlap them slightly.

	Bevel_overlap controls the fractional overlap of the two bevels,
	and delay specifies an extra delay for b.

	If ta and/or tb are specified, return a tuple of
	(concatenated_time_series, tma, tmb) where tma and tmb are the locations
	corresponding to ta and tb in the corresponding input arrays.
	"""
	assert bevel >= 0
	if bevel > 0:
		bev = (0.5 + N.arrayrange(bevel))/float(bevel)
	else:
		bev = 1
	bc = N.array(b, copy=True)
	bev = (0.5 + N.arrayrange(bevel))/float(bevel)
	ans = a.shape[0]-1
	bns = bc.shape[0]-1
	bos = a.shape[0] - int(round(bevel_overlap*bevel)) + delay
	boe = bos + bc.shape[0]
	o = N.zeros((boe,), N.Float)
	o[:a.shape[0]] = a
	if bevel > 0:
		N.multiply(o[:bevel], bev, o[:bevel])
		N.multiply(o[ans:ans-bevel:-1], bev, o[ans:ans-bevel:-1])
		N.multiply(bc[:bevel], bev, bc[:bevel])
		N.multiply(bc[bns:bns-bevel:-1], bev, bc[bns:bns-bevel:-1])
	N.add(o[bos:boe], bc, o[bos:boe])
	if ta is not None or tb is not None:
		if tb is not None:
			tb += bos
		return (o, ta, tb)
	else:
		return o


def argmax(a):
	i = N.argmax(a, axis=None)
	o = []
	for n in reversed(a.shape):
		o.append( i % n )
		i = i//n
	o.reverse()
	return tuple(o)


def _test_argmax():
	x = N.array([1,2,3,4,3])
	assert argmax(x) == (3,)
	x = N.array([[1,2,3,4,3], [0,1,0,1,0]])
	assert x[argmax(x)] == 4
	x = N.array([[1,2,3,4,3], [0,1,0,5,0]])
	assert x[argmax(x)] == 5
	x = N.array([[1,2], [3,4], [5,6], [0,1],[0,5],[0,7]])
	assert x[argmax(x)] == 7
	x = N.zeros((5,4,3,3,1,2), N.Float)
	x[2,1,0,1,0,1] = 100.0
	assert argmax(x) == (2,1,0,1,0,1)
	x[0,0,0,0,0,0] = 200.0
	assert argmax(x) == (0,0,0,0,0,0)
	x[4,3,2,2,0,1] = 300.0
	assert argmax(x) == (4,3,2,2,0,1)
	x[4,3,2,2,0,0] = 400.0
	assert argmax(x) == (4,3,2,2,0,0)
	x[4,3,2,0,0,0] = 500.0
	assert argmax(x) == (4,3,2,0,0,0)


def N_maximum(a):
	assert len(a.shape) == 1
	return a[N.argmax(a)].item()


def N_minimum(a):
	assert len(a.shape) == 1
	return a[N.argmin(a)].item()


# Median along the first axis.
N_median = N.median


def N_frac_rank(a, fr):
	assert 0 <= fr <= 1.0
	tmp = N.sort(a)
	n = tmp.shape[0]
	assert n > 0, "Zero-sized array: cannot compute rank."
	return tmp[int(round((n-1)*fr))].item()


def N_mean_ad(a):
	"""Mean absolute deviation.   For a multi-dimensional array,
	it takes the MAD along the first axis, so
	N_mean_ad(x)[0]==N_mean_ad(x[:,0]).
	"""
	ctr = N.median(a)
	diff = N.subtract(a, ctr)
	return N.sum(N.absolute(diff), axis=0)/(diff.shape[0]-1)


def _test_N_mean_ad():
	x = N.zeros((2,1), N.Float)
	x[0,0] = 1
	x[1,0] = 0
	assert N.allclose(N_mean_ad(x), [1.0])
	x = N.zeros((5,7), N.Float)
	x[0,0] = 1
	y = N_mean_ad(x)
	assert y.shape == (7,)
	assert N.allclose(y, [(1.0-0.0)/4.0, 0, 0, 0, 0, 0, 0])
	assert abs(N_mean_ad(x)[0]-N_mean_ad(x[:,0])) < 0.001


def median_across(list_of_vec):
	"""Returns an element-by-element median of a list of Numeric vectors."""
	assert len(list_of_vec[0].shape) == 1
	o = N.zeros(list_of_vec[0].shape, N.Float)
	tmp = N.zeros((len(list_of_vec),), N.Float)
	for t in list_of_vec:
		assert t.shape == o.shape
	for i in range(o.shape[0]):
		for (j, v) in enumerate(list_of_vec):
			tmp[j] = v[i]
		o[i] = N_median(tmp)
	return o



if hasattr(N, 'var'):
	variance = N.var
else:
	def variance(x):
		BLOCKSIZE = 20000
		x = N.asarray(x, N.Float)
		n = x.shape[0]
		x0 = x[0]
		s = 0.0
		ss = 0.0
		for i in range(0, x.shape[0], BLOCKSIZE):
			e = min(n, i+BLOCKSIZE)
			tmp = x[i:e] - x0
			s += N.sum(tmp, axis=0)
			ss += N.sum(tmp**2, axis=0)
		return N.maximum(ss-s**2/n, 0.0)/(n-1)


if hasattr(N, 'std'):
	stdev = N.std
else:
	def stdev(x):
		return math.sqrt(variance(x))


if hasattr(N, 'diag'):
	make_diag = N.diag
else:
	def make_diag(x):
		"""Construct a square diagonal matrix whose elements
		are taken from the vector x.
		"""
		x = N.asarray(x, N.Float)
		assert len(x.shape) == 1
		n = x.shape[0]
		o = N.zeros((n,n), N.Float)
		for i in range(n):
			o[i,i] = x[i]
		return o


def set_diag(x, a):
	"""Set the diagonal of a matrix x to be the vector a.
	If a is shorter than the diagonal of x, just set the beginning."""

	assert len(a.shape) == 1
	assert len(x.shape) == 2
	n = a.shape[0]
	assert x.shape[0] >= n
	assert x.shape[1] >= n
	for i in range(n):
		x[i,i] = a[i]


# Take the median along the first coordinate of a 2-D matrix,
# so if m.shape=(2,4) then the output will have shape (4,)
N_median_across = N.median


def _test_N_median_across():
	x = N.zeros((3,2), N.Float)
	x[0,0] = 1
	x[1,0] = 2
	x[2,0] = 3
	y = N_median_across(x)
	assert N.allclose(y, [2.0, 0.0])


if hasattr(N, 'clip'):
	def limit(low, x, high):
		return N.clip(x, low, high)
else:
	def limit(low, x, high):
		"""Force x to be in the range [low, high]."""
		assert high >= low, "Impossible limit: high=%s is not >= low=%s." % (str(high), str(low))
		if x < low:
			return low
		if x > high:
			return high
		return x



def trimmed_mean_sigma_across(list_of_vec, weights, clip):
	import gpkavg
	"""Returns an element-by-element trimmed_mean of a list of Numeric vectors.
	For instance, the first component of the output vector is the trimmed mean
	of the first component of all of the input vectors.
	"""
	import gpkavg
	assert len(list_of_vec[0].shape) == 1
	om = N.zeros(list_of_vec[0].shape, N.Float)
	osig = N.zeros(list_of_vec[0].shape, N.Float)
	tmp = N.zeros((len(list_of_vec),), N.Float)
	for t in list_of_vec:
		assert t.shape == om.shape
	for i in range(om.shape[0]):
		for (j, v) in enumerate(list_of_vec):
			tmp[j] = v[i]
		om[i], osig[i] = gpkavg.avg(tmp, weights, clip)
	return (om, osig)

def trimmed_mean_across(list_of_vec, weights, clip):
	return trimmed_mean_sigma_across(list_of_vec, weights, clip)[0]

def trimmed_stdev_across(list_of_vec, weights, clip):
	return trimmed_mean_sigma_across(list_of_vec, weights, clip)[1]


def vec_variance(x):
	"""Take a component-by-component variance of a list of vectors."""
	n = len(x)
	if n < 2:
		raise ValueError, "Cannot take variance unless len(x)>1"
	x0 = x[0]
	sh = x0.shape
	s = N.zeros(sh, N.Float)
	ss = N.zeros(sh, N.Float)
	for xi in x:
		assert xi.shape == sh
		tmp = xi - x0
		N.add(s, tmp, s)
		N.add(ss, tmp**2, ss)
	return N.maximum(ss-s**2/n, 0.0)/(n-1)




def qform(vec, mat):
	"""A quadratic form: vec*mat*vec,
	or vecs*mat*transpose(vecs)"""
	if len(vec.shape) != 1 or len(mat.shape) != 2:
		raise ValueError, ': '.join(["Can't handle input",
					     "requires vector*matrix(vector)",
					     "shapes are %s and %s" % (
							vec.shape, mat.shape)
						])
	if mat.shape != (vec.shape[0], vec.shape[0]):
		raise ValueError, ': '.join([
					"Matrix must be square and match the length of vector",
					"shapes are %s and %s" % (vec.shape, mat.shape)
					])
	return N.dot(vec, N.matrixmultiply(mat, vec))



def KolmogorovSmirnov(d1, d2, w1=None, w2=None):
	d1 = N.asarray(d1)
	d2 = N.asarray(d2)
	assert len(d1.shape) == 1, 'd1.shape=%s' % str(d1.shape)
	assert len(d2.shape) == 1, 'd2.shape=%s' % str(d2.shape)
	if w1 is None:
		w1 = N.ones(d1.shape, N.Float)
	if w2 is None:
		w2 = N.ones(d2.shape, N.Float)
	ws1 = N.sum(w1)
	ws2 = N.sum(w2)
	i1 = N.argsort(d1)
	i2 = N.argsort(d2)
	c1 = 0.0
	c2 = 0.0
	j1 = 0
	j2 = 0
	maxdiff = 0.0
	while True:
		if abs(c1-c2) > maxdiff:
			maxdiff = abs(c1 - c2)
		if j1 < i1.shape[0]-1 and j2<i2.shape[0]-1:
			if d1[i1[j1]] < d2[i2[j2]]:
				j1 += 1
				c1 += w1[i1[j1]]/ws1
			elif d1[i1[j1]] > d2[i2[j2]]:
				j2 += 1
				c2 += w2[i2[j2]]/ws2
			else:
				j1 += 1
				c1 += w1[i1[j1]]/ws1
				j2 += 1
				c2 += w2[i2[j2]]/ws2
		elif j1==i1.shape[0]-1 and j2==i2.shape[0]-1:
			break
		elif j1 < i1.shape[0]-1:
			j1 += 1
			c1 += w1[i1[j1]]/ws1
		else:
			j2 += 1
			c2 += w2[i2[j2]]/ws2
	return maxdiff


def _testKS():
	assert KolmogorovSmirnov([1, 2, 3.01, 3, 4, 5], [1, 2, 3.01, 3, 4, 5]) < 0.001
	assert abs( KolmogorovSmirnov([1, 2, 3.01, 3, 4, 5], [1, 2, 3, 4, 5, 6]) - 0.16667) < 0.001
	print KolmogorovSmirnov([1, 2, 3.01, 3, 4, 5], [1, 2, 3, 4, 5])




def interpN(a, t):
	"""Interpolate array a to floating point indices, t,
	via nearest-neighbor interpolation.
	Returns a Numeric array.
	"""
	ii = N.around(t)
	n = a.shape[0]
	if not N.alltrue((ii>=0) * (ii<=n-1)):
		raise IndexError, "Index out of range."
	iii = ii.astype(N.Int)
	return N.take(a, iii, axis=0)



def interp(a, t):
	"""Interpolate to a specified time axis.
	This does a linear interpolation.
	A is a Numpy array, and t is an array of times.
	Returns a Numpy array.
	"""
	n = a.shape[0]
	nt = t.shape[0]
	las = len(a.shape)
	# print 'a=', a.shape, a
	if las == 1:
		# print 'a.shape=', a.shape
		a = N.transpose([a], N.Float)
	m = a.shape[1]
	ii = N.around(t)
	assert ii.shape == t.shape

	# print 't=', t
	# print 'n=', n, 'ii=', ii
	if not N.alltrue((ii>=0) * (ii<=n-1)):
		raise IndexError, "Index out of range."

	nearestT = ii.astype(N.Int)
	assert nearestT.shape == t.shape

	nearestA = N.take(a, nearestT, axis=0)
	assert nearestA.shape == (nt,m)
	deltaT = t - nearestT
	assert deltaT.shape == nearestT.shape
	isupport = N.where((deltaT>=0)*(nearestT<n-1)+(nearestT<=0), 1, -1) + nearestT
	assert isupport.shape == (nearestA.shape[0],)
	assert isupport.shape == nearestT.shape
	assert isupport.shape == deltaT.shape
	# print 'a=',a 
	# print 'isupport=', isupport
	support = N.take(a, isupport, axis=0)
	assert support.shape == (nt,m)
	if len(a.shape) > 1:
		assert nearestA.shape == (nt,m)
		deltaA = (deltaT/(isupport-nearestT))[:,N.NewAxis] * (support-nearestA)
		assert deltaA.shape[1] == a.shape[1]
	else:
		deltaA = (deltaT/(isupport-nearestT)) * (support-nearestA)
	assert deltaA.shape == nearestA.shape
	rv = nearestA + deltaA
	if las == 1:
		return rv[:,0]
	return rv


def _test_interp1():
	a = N.array([[1.0], [2.0]])
	# a = N.array([1.0, 2.0], N.Float)
	t = N.array([0.5])
	q = interp(a, t)
	assert q.shape == (1,1)
	assert N.sum(N.absolute(interp(a, t) - [1.5])) < 1e-3


def split_into_clumps(x, threshold, minsize=1):
	"""This reports when the signal is above the threshold.
	@param x: a signal
	@type x: L{numpy.ndarray}, one-dimensional.
	@param threshold: a threshold.
	@type threshold: float
	@returns: [(start, stop), ...] for each region ("clump") where C{x>threshold}.
	"""
	assert len(x.shape) == 1
	gt = N.greater(x, threshold)
	chg = gt[1:] - gt[:-1]
	nz = N.nonzero(chg)[0]
	rv = []
	if gt[0]:
		last = 0
	else:
		last = None
	for i in nz:
		if last is None:
			last = i+1
		else:
			if minsize > 0:
				rv.append( (last, i+1) )
			last = None
	if last is not None and gt.shape[0]>=last+minsize:
		rv.append( (last, gt.shape[0]) )
	return rv


def _test_split_into_clumps():
	tmp = split_into_clumps(N.array([0,0,0,1,0,0]), 0.5)
	assert tmp == [(3,4)]
	tmp = split_into_clumps(N.array([1,0,0]), 0.5)
	assert tmp == [(0,1)]
	tmp = split_into_clumps(N.array([0,0,0,1]), 0.5)
	assert tmp == [(3,4)]
	tmp = split_into_clumps(N.array([1]), 0.5)
	assert tmp == [(0,1)]
	tmp = split_into_clumps(N.array([0]), 0.5)
	assert tmp == []
	tmp = split_into_clumps(N.array([1]), 0.5, minsize=2)
	assert tmp == []
	tmp = split_into_clumps(N.array([0,0,0,1,1,0]), 0.5, minsize=2)
	assert tmp == [(3,5)]


if __name__ == '__main__':
	_test_split_into_clumps()
	_test_argmax()
	_test_interp1()
	_test_Poisson()
	_test_N_mean_ad()
	_test_N_median_across()




"""This returns the entropy of a probability distribution that
produced a given sample.
"""

import Num
import mcmc
import mcmc_helper
import gpkavg
import math

import kl_dist


def entropy_probs(p):
	"""Entropy of a probability distribution."""
	rv = Num.sum( p*Num.log(p) ) / math.log(2)
	return rv


def entropy_vec(p, N=None, F=1.0, Clip=0.01):
	"""Entropy of a frequency distribution p.
	Here, we assume that p is counts
	derived from multinomial distributed data;
	they are not normalized to one.
	"""

	p = Num.asarray(p, Num.Int)
	if N is None:
		N = p.shape[0]**2 * 30
	assert Num.sum(p) > 0
	pstart = ((0.5+p)/Num.sum(0.5+p))
	pV = 0.1*Num.identity(p.shape[0])/float(p.shape[0])**1.5
	xp = mcmc.bootstepper(kl_dist.multinomial_logp, pstart, pV,
				c=(p,F), fixer=kl_dist.multinomial_fixer)
	mcmch = mcmc_helper.stepper(xp)
	mcmch.run_to_bottom()
	mcmch.run_to_ergodic(5.0)
	o = []
	while len(o) < N:
		mcmch.run_to_ergodic(1.0/math.sqrt(N))
		o.append( entropy_probs( kl_dist.P(xp.prms()) ) )
	avg, sigma = gpkavg.avg(o, None, Clip)
	return (-avg, sigma)



if __name__ == '__main__':
	print entropy_vec([100,100,100.0,100.0])



import struct
import sys

import Numeric


# Class for loading mfccfiles in the C_SDK format. Supports byteswapping.
# Questions: Tor Andre Myrvoll, myrvoll@research.bell-labs.com

class MFCCFile:
    # This class loads a mfcc file in LASR format and gives access to
    # mfcc vectors indexed by file and position
    def __init__(self,filename,DEBUG = 0,BYTEORDER='@'):
        self.DEBUG = DEBUG
        self.BYTEORDER = BYTEORDER
        self.FileName = filename
        long_s = 4
        float_s = 4 

        # See if we should byteswap
        if not struct.pack('%sL' % (self.BYTEORDER),1) == struct.pack('L',1):
            BYTESWAP = 1
        else:
            BYTESWAP = 0

        fp = open(filename,'r')
        if DEBUG:
            sys.stderr.write("Opening file %s\n" % (filename))
            


        # Store the number of sentences in this file
        str = fp.read(long_s)
        self.numSent = struct.unpack('%sL' % (self.BYTEORDER),str)[0]
        if DEBUG:
            sys.stderr.write("    Number of sentences: %d\n" % (self.numSent))

        # Store the feature vector dimension
        str = fp.read(long_s)
        self.vecSize = struct.unpack('%sL' % (self.BYTEORDER),str)[0]
        if DEBUG:
            sys.stderr.write("    Vector size: %d\n" % (self.vecSize))
            

        # Load an array of sentence lengths into a list
        self.sentLength = []
        for sent in range(self.numSent):
            str = fp.read(long_s)
            length = struct.unpack('%sL' % (self.BYTEORDER),str)[0]
            self.sentLength.append(length)
        if DEBUG:
            sys.stderr.write("\n")
            

        # Now load the entire file and store the vectors in lists
        self.sentList = []
        sentnum = 0
        index = 0
        veclength = self.vecSize*float_s
        if DEBUG:
            sys.stderr.write("    Loading sentences:")
            

        str = fp.read()
        for sent in range(self.numSent):
            if DEBUG:
                sys.stderr.write(" %d" % (sent))
            sentdata = []
            for vec in range(self.sentLength[sent]):
                mfccvec = Numeric.fromstring(str[index:index+veclength],'f')
                if BYTESWAP:
                    mfccvec = mfccvec.byteswapped()
                sentdata.append(mfccvec)
                index += veclength
                
            self.sentList.append(sentdata)

        # We're done. Close the file
        fp.close()
        if DEBUG:
            sys.stderr.write("\n")

    def Save(self,filename):
        fh = open(filename,'w')
        # First we write the header
        str = struct.pack('LL',self.numSent,self.vecSize)
        fh.write(str)

        # Now the array of sentence lengths
        for length in self.sentLength:
            str = struct.pack('L',length)
            fh.write(str)

        # Finally, we save the cepstral vectors themselves
        for utt in self.sentList:
            for vec in utt:
                str = vec.tostring()
                fh.write(str)

        fh.close()
            
    def ReturnSent(self,sentnum):
        # Returns a list containing the entire sentence
        return self.sentList[sentnum]
    

    def ReturnVector(self,sentnum,vecnum):
        # Returns vector i of utterance j
        return self.sentList[sentnum][vecnum]
    



import Num
import gpk_lsq
import gpkavg


class NotEnoughData(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)


class w_linear_least_squares(gpk_lsq.linear_least_squares):
	def __init__(self, a, y, w, minsv=None, minsvr=None, copy=True):
		a = Num.asarray(a, Num.Float)
		if len(a.shape) != 2:
			raise ValueError, "a needs to be 2-dimensional: shape=%s" % str(a.shape)

		self.w = Num.array(w, Num.Float, copy=copy)
		if len(self.w.shape) != 1 and self.w.shape[0] != a.shape[0]:
			raise ValueError, "w needs to be 1-dimensional and match a: shape=%s vs %s" % (str(self.w.shape), str(a.shape))

		aw = self.w[:,Num.NewAxis] * a
		gpk_lsq.linear_least_squares.__init__(self, aw, y, minsv=minsv,
								minsvr=minsvr, copy=copy)

	def fit(self, copy=False):
		return gpk_lsq.linear_least_squares.fit(self, copy=False)/self.w

	def y(self, copy=True):
		return gpk_lsq.linear_least_squares.y(self, copy=False)/self.w

	def set_y(self, y, copy=True):
		if y is not None:
			y = self.w * y
		gpk_lsq.linear_least_squares.set_y(self, y, copy=False)
		




class robust_linear_fit(w_linear_least_squares):
	HTMIN = 3.0
	STMIN = 1.5

	def __init__(self, a, y, sigma_y=None, minsv=None, minsvr=None):
		"""This does bounded influence regression, with
		a robust M-estimator in the y-direction.
		"""

		y = Num.asarray(y, Num.Float)
		assert len(y.shape) == 1
		ndat = y.shape[0]
		if sigma_y is None:
			sigma_y = 1.0
		if isinstance(sigma_y, (float, int)):
			self.sigma_y = sigma_y * Num.ones(y.shape, Num.Float)
		else:
			self.sigma_y = Num.asarray(sigma_y, Num.Float)
			assert len(self.sigma_y.shape) == 1
			assert self.sigma_y.shape[0] == ndat

		a = Num.asarray(a, Num.Float)
		assert len(a.shape) == 2
		assert a.shape[0] == ndat
		
		if a.shape[1] >= ndat:
			raise NotEnoughData, "Not enough data to bound influence function"

		w = 0
		w_hat = Num.ones(y.shape, Num.Float)
		w_resid = Num.ones(y.shape, Num.Float)
		hthresh = 6*self.HTMIN
		sthresh = 6*self.STMIN
		endgame = False
		while 1:
			last_w = w
			w = w_hat * w_resid
			if Num.alltrue( Num.absolute(w-last_w) < 0.01 ):
				if endgame:
					break
				else:
					hthresh = self.HTMIN
					sthresh = self.STMIN
					endgame = True
					# print 'ENDGAME'

			# print 'w=', w
			# print 'y=', y

			self.w = w
			wsig = w/self.sigma_y

			w_linear_least_squares.__init__(self, a, y, wsig, minsv=minsv, minsvr=minsvr)

			ht = (1 - 2.0/y.shape[0])/hthresh
			# print 'hat=', self.hat()
			if Num.sometrue(self.hat() >= 1.0):
				raise NotEnoughData, "Cannot bound influence of point(s)."
			hatfac = (1-self.hat())/ht
			# print 'hatfac=', hatfac
			w_hat = Num.minimum(1.0, w_hat*hatfac**0.5)
			# Keep largest weight at unity:
			w_hat = w_hat/w_hat[Num.argmax(w_hat)]
			# print 'w_hat=', w_hat

			normerr = (y - self.fit())/self.sigma_y
			# print 'normerr=', normerr
			self.typdev = gpkavg.avg(Num.absolute(normerr), None, 0.25)[0]
			r_bar = self.typdev * sthresh
			# print 'rbar=', r_bar
			w_resid = 0.5*(w_resid + Num.hypot(1, normerr/r_bar)**(-1.0) )
			wrs = Num.sqrt( Num.sum(w_resid**2, axis=0)/ndat )
			Num.divide(w_resid, wrs, w_resid)
			# print 'w_resid=', w_resid

			hthresh = self.HTMIN + 0.5*(hthresh-self.HTMIN)
			sthresh = self.STMIN + 0.5*(sthresh-self.STMIN)
			# print 'ht, st=', hthresh, sthresh
		# print "END"



def test_w1():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 100], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.array([0.0, 1, 2, 3, 4, 5, 6, 7, 18.01, 99])
	w = Num.array([1, 3.2, 1, 1, 5, 1, 1, 1, .0001, 1])
	soln = w_linear_least_squares(basis, y, w)
	print 'soln.x', soln.x()
	print 'soln.fit', soln.fit()
	assert len(soln.x().shape) == 1
	assert len(soln.fit().shape) == 1
	assert soln.rank() == 2
	print 'residual=', soln.residual()
	assert Num.sum(Num.absolute(soln.residual()) > 0.2) == 1
	assert Num.sum(Num.absolute(soln.residual()) > 10.2) == 0
	err = Num.sum((soln.fit()-y)**2)
	assert err>=0
	assert Num.sum(err > 0.02) == 1
	assert soln.sv().shape == (2,)
	assert abs(soln.x()[0] - 1) < 0.01
	assert abs(soln.x()[1] + 1) < 0.01
	assert Num.sum(Num.absolute(w-soln.w)) < 1e-6
	assert Num.sum(Num.absolute(Num.dot(basis, soln.x())-soln.fit())) < 1e-6



def test_w1b():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 100], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.array([0.01, 1, 2.01, 3, 3.99, 5, 6.01, 7, 88.01, 99])
	w = Num.array([0.2, 1, 0.2, 5, 1, 0.2, 1, 2, .001, 0.01])
	soln = w_linear_least_squares(basis, y, w)
	print 'soln.x', soln.x()
	print 'soln.resid', soln.residual()
	assert Num.sum(Num.absolute(soln.residual()) > 0.05) == 1
	assert abs(soln.x()[0] - 1) < 0.01
	assert abs(soln.x()[1] + 1) < 0.01
	print 'sf', soln.fit()
	print 'mm', Num.matrixmultiply(basis, soln.x())
	assert Num.sum(Num.absolute(Num.matrixmultiply(basis, soln.x())
							-soln.fit())
				) < 1e-6

def test_r1():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 100], [1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.array([0.01, 1, 1.99, 3, 4.01, 5, 5.99, 7, 88.01, 99])
	soln = robust_linear_fit(basis, y, 1)
	print 'soln.x', soln.x
	assert len(soln.x().shape) == 1
	assert len(soln.fit().shape) == 1
	assert soln.rank() == 2
	err = Num.sum((soln.fit()-y)**2)
	assert err>=0
	assert Num.sum(err > 0.02) == 1
	assert soln.sv().shape == (2,)
	assert abs(soln.x()[0] - 1) < 0.02
	assert abs(soln.x()[1] + 1) < 0.02
	assert Num.sum(Num.absolute(Num.matrixmultiply(basis, soln.x())
						-soln.fit())
				) < 1e-5

def test_r1hat():
	basis = Num.transpose([[1, 2, 3, 4, 5, 6, 7, 8, 9, 100],
				[1, 1, 1, 1, 1, 1, 1, 1, 1, 1]])
	y = Num.array([0.01, 1, 1.99, 3, 4.01, 5, 5.99, 7, 8, 90])
	soln = robust_linear_fit(basis, y, 1)
	print 'soln.x', soln.x
	assert soln.rank() == 2
	print 'resid=', soln.residual()
	assert Num.sum(Num.absolute(soln.residual()) > 0.04) == 1
	assert abs(soln.x()[0] - 1) < 0.05
	assert abs(soln.x()[1] + 1) < 0.05
	assert Num.sum(Num.absolute(Num.matrixmultiply(basis, soln.x())
						- soln.fit())
			) < 1e-5


if __name__ == '__main__':
	test_w1()
	test_w1b()
	test_r1()
	test_r1hat()



"""This is a helper module to make use of mcmc.py and mcmc_big.py.
It allows you to conveniently run a Monte-Carlo simulation of any
kind until it converges (L{stepper.run_to_bottom}) or until it
has explored a large chunk of parameter space (L{stepper.run_to_ergodic}).

It also helps you with logging the process.
"""

from __future__ import with_statement
import sys
import thread as Thr
from gmisclib import Num
from gmisclib import g_implements
from gmisclib import mcmc

Debug = 0

class TooManyLoops(Exception):
	def __init__(self, *s):
		Exception.__init__(self, *s)


class warnevery(object):
	def __init__(self, interval):
		self.c = 0
		self.interval = interval
		self.lwc = 0
	
	def inc(self, incr=1):
		self.c += incr
		if self.c > self.lwc + self.interval:
			self.lwc = self.c
			return 1
		return 0

	def count(self):
		return self.c


class _counter(object):
	def __init__(self):
		self.lock = Thr.allocate_lock()
		self.count = 0

	def update(self, accepted):
		with self.lock:
			self.count += accepted

	def get(self):
		with self.lock:
			tmp = self.count
		return tmp



class _accum_erg(object):
	def __init__(self):
		self.lock = Thr.allocate_lock()
		self.erg = 0.0
		self.last_resetid = -1

	def update(self, x):
		"""Increment an accumulator, but reset it to
		zero if the underlying stepper has called reset().
		The increment is by the ergodicity estimate, so
		C{self.erg} should approximate how many times the
		MCMC process had wandered across the probability
		distribution.
		"""
		with self.lock:
			tmp = x.reset_id()
			if tmp != self.last_resetid:
				self.erg = 0.0
				self.last_resetid = tmp
			else:
				self.erg += x.ergodic()


	def get(self):
		with self.lock:
			tmp = self.erg
		return tmp


class logger_template(object):
	def add(self, stepperInstance, iter):
		pass

	def close(self):
		pass




class stepper(object):
	def __init__(self, x, maxloops=-1, logger=None):
		self.logger = None	# This sets things up for close()
		self.iter = 0
		self.maxloops = maxloops
		g_implements.check(x, mcmc.stepper)
		self.x = x
		if logger is None:
			self.logger = logger_template()
		else:
			g_implements.check(logger, logger_template)
			self.logger = logger
		self.OOZE = 0.9


	def reset_loops(self, maxloops=-1):
		self.maxloops = maxloops


	def run_to_change(self, ncw=1, update_T=False):
		"""Run the X{Markov Chain Monte-Carlo} until it finds
		an acceptable step.

		@param update_T: Permission to fiddle with the simulated
			annealing temperature.   If C{update_T} is False,
			C{self.x.T} is left with whatever value you
			originally gave it.
		@type update_T: boolean.
		"""
		assert 0 <= self.OOZE < 1.0
		ooze = self.OOZE**(1.0/self.x.np)
		T = self.x.T
		n = 0
		self.communicate_hook()
		while n < ncw:
			if update_T:
				pre_logp = self.x.current().logp_nocompute()
			acc = self.x.step()
			if acc and update_T:
				logp = self.x.current().logp_nocompute()
				T = max(0.0, T*ooze + 0.4*(logp-pre_logp))
				self.x.T = max(1.0, T)
			n += acc
			self.logger.add(self.x, self.iter)
			if self.maxloops > 0:
				self.maxloops -= 1
			if self.maxloops == 0:
				raise TooManyLoops, "run_to_change"
			self.iter += 1
		self.join('rtc end')
		return self.x.current()


	def communicate_hook(self):
		return


	def close(self):
		"""After calling close(), it is no longer legal
		to call run_to_change(), run_to_ergodic(), or
		run_to_bottom().    Logging is shut down, but the
		contents of the stepper are still available for
		inspection.
		"""
		if self.logger is not None:
			self.logger.close()
			self.logger = None


	# def __del__(self):
		# self.close()


	def _not_yet_ergodic(self, nc, ncw):
		return nc.get() < ncw


	def run_to_ergodic(self, ncw=1, T=1.0):
		"""Run the stepper until it has explored all of parameter space
		C{ncw} times (as best as we can estimate).   Note that this
		is a pretty careful routine.   If the stepper finds a better
		region of parameter space so that log(P) improves part way
		through the process, the routine will reset itself and begin
		again.

		NOTE: this sets C{T} and C{sortstrategy} for the L{stepper<mcmc.BootStepper>}.

		@param ncw: how many times (or what fraction) of an ergodic
			exploration to make.
		@type ncw: int.
		@param T: temperature at which to run the Monte-Carlo process.
			This is normally 1.0.   If it is C{None}, then the
			current temperature is not modified.
		@type T: C{float} or C{None}.
		@return: a L{position<mcmc.position_base>} at the end of the process.
		"""
		self.join('rte before loop')
		if T is not None:
			self.x.T = T
		ss = self.x.set_sort_strategy(self.x.SSNEVER)
		nc = _accum_erg()
		while self._not_yet_ergodic(nc, ncw):
			# self.join('rte loop top')
			self.communicate_hook()
			for i in range(4):
				self.x.step()
				self.logger.add(self.x, self.iter)
				nc.update(self.x)
				self.iter += 1
				if self.maxloops > 0:
					self.maxloops -= 1
				if self.maxloops == 0:
					raise TooManyLoops, ("run_to_ergodic", nc.get(), ncw)
		self.x.set_sort_strategy(ss)	# Return to previous sortstrategy.
		self.join('rte after loop')
		return self.x.current()




	def _not_at_bottom(self, xchanged, nchg, es, dotchanged, ndot, update_T):
		return (Num.sometrue(Num.less(xchanged,nchg))
					or es<1.0 or dotchanged<ndot
					or (update_T and self.x.T>1.5)
				)


	def run_to_bottom(self, ns=3, update_T=True):
		"""Run the X{Markov Chain Monte-Carlo} until it converges
		near a minimum.

		@param ns: This controls how carefully if checks that
			it has really found a minimum.   Large values
			will take longer but will assure more accurate
			convergence.
		@type ns: L{int}
		@param update_T: Permission to fiddle with the simulated
			annealing temperature.   If C{update_T} is False,
			C{self.x.T} is left with whatever value you
			originally gave it.
		@type update_T: boolean.
		"""
		m = self.x.np
		nchg = int(round(ns * m**0.25))
		ndot = int(round((2*ns)**0.5))
		for i in range(2+m//2):
			self.run_to_change(4, update_T=update_T)	# get it warmed up.
		before = self.run_to_change(ns, update_T=update_T)
		mid = self.run_to_change(ns, update_T=update_T)
		es = 0.0
		lock = Thr.allocate_lock()
		xchanged = Num.zeros(m, Num.Int)
		dotchanged = 0
		last_resetid = self.x.reset_id()
		while self._not_at_bottom(xchanged, nchg, es, dotchanged, ndot, update_T):
			try:
				after = self.run_to_change(ns, update_T=update_T)
			except TooManyLoops, x:
				raise TooManyLoops, ('run_to_bottom: s' % x, self.iter, es, dotchanged, m,
							Num.sum(Num.less(xchanged, nchg)),
							self.x.current().prms()
							)
			# We keep track of direction reversals to see when
			# we reach the bottom.
			Num.add(xchanged,
				Num.less( (after.vec()-mid.vec())
						* (mid.vec()-before.vec()), 0.0),
				xchanged)

			dotchanged += Num.dot(after.vec()-mid.vec(),
						mid.vec()-before.vec()) < 0.0

			es += self.x.ergodic() * ns**0.5

			# print 'LA'
			lock.acquire()
			if self.x.reset_id() != last_resetid:
				# A new maximum: we are probably
				# not stable.  Likely, we are still in the
				# improvement phase.
				last_resetid = self.x.reset_id()
				lock.release()
				es = 0.0
				xchanged[:] = 0
				dotchanged = 0
			else:
				lock.release()

			before = mid
			mid = after
		self.join('rtb after loop')
		return self.iter


	def join(self, id):
		return

	def note(self, s, lvl):
		if Debug >= lvl:
			print s
			sys.stdout.flush()


def is_root():
	"""This is a stub for compatibility with MPI code."""
	return True

def size():
	"""This is a stub for compatibility with MPI code."""
	return 1


def test():
	def test_logp(x, c):
		# print '#', x[0], x[1]
		return -(x[0]-x[1]**2)**2 + 0.001*x[1]**2
	x = mcmc.bootstepper(test_logp, Num.array([0.0,2.0]),
				Num.array([[1.0,0],[0,2.0]]))
	thr = stepper(x)
	# nsteps = thr.run_to_bottom(x)
	# print '#nsteps', nsteps
	# assert nsteps < 100
	for i in range(2):
		print 'RTC'
		thr.run_to_change(2)
		print 'RTE'
		thr.run_to_ergodic(1.0)
		print 'DONE'
	thr.close()




if __name__ == '__main__':
	test()




"""Solve a 1-dimensional equation to find the roots.
This does alternating secant and bisection steps.
It will work for any function, discontinuous or not.
For a nasty function, it might be a factor of 2 slower
than bisection, but for a nearly-linear function,
it can be much faster than bisection.
"""

import die


def _root1(f, xl, xh, p, fl, fh, epsx, epsf, dbg=None):
	if dbg:
		dbg.append( ('xl:%s sh:%s fl:%s fh:%s', (xl, xh, fl, fh)) )
	# print "(%f, %f) -> (%f, %f)" % (xl, fl, xh, fh)
	assert epsx >= 0.0
	assert epsf >= 0.0
	assert xh > xl
	if fl == 0:
		if dbg:
			dbg.append( ('fl==0, return xl= %s', repr(xl)) )
		return xl
	if fh == 0:
		if dbg:
			dbg.append( ('fh==0, return xh= %s', repr(xh)) )
		return xh
	if xh-xl<=epsx or abs(fl)<epsf or abs(fh)<epsf:
		tmp = xl + (xl-xh)*fl/(fh-fl)  #  (xl*fh-xh*fl)/(fh-fl)
		assert xl <= tmp <= xh, "Not xl=%.15g <= tmp=%.15g <= xh=%.15g" % (xl, tmp, xh)
		if dbg:
			dbg.append( ('xl:%s sh:%s fl:%s fh:%s tmp:%s', (xl, xh, fl, fh, tmp)) )
		if tmp > xh:
			if dbg:
				dbg.append( ('Return xh', () ) )
			return xh
		elif tmp < xl:
			if dbg:
				dbg.append( ('Return xl', () ) )
			return xl
		else:
			if dbg:
				dbg.append( ('Return tmp', () ) )
			return tmp
		

	assert (fh>0) != (fl>0)
	q = [ (xl, fl), (xh, fh) ]

	# Do a secant step...
	xsec = (xl*fh-xh*fl)/(fh-fl)
	if dbg:
		dbg.append( ('xsec=%s', xsec) )
	if xsec>xl and xsec<xh:
		fsec = f(xsec, p)
		if fsec == 0:
			if dbg:
				dbg.append( ('Fsec==0, xsec=%s', xsec) )
			return xsec
		# print "secant: (%f, %f)" % (xsec, fsec)
		q.append( (xsec, fsec) )

		if (fsec>0) == (fl>0):
			xbi = xsec + (xh-xsec)*0.5
		else:
			xbi = xsec + (xl-xsec)*0.5
	else:
		xbi = xl + 0.5*(xh - xl)
	if dbg:
		dbg.append( ('xbi=%s', xbi) )

	# ...followed by a bisection-like step.

	assert xl <= xbi <= xh, "Not xl=%.15g <= xbi=%.15g <= xh=%.15g" % (xl, xbi, xh)
	if xbi > xl and xbi < xh and xbi!=xsec:
		fbi = f(xbi, p)
		if dbg:
			dbg.append( ('fbi=%s', fbi) )
		q.append( (xbi, fbi) )

	if len(q) == 2:	# No progress is being made, presumably
			# because xl and xh are neighbouring reals.
		# print "RETURN no progress"
		if dbg:
			dbg.append( ('len(q)=%d', len(q)) )
		return 0.5*(xl+xh)

	q.sort()

	X = 0
	F = 1
	# Then, we see which interval contains the root, and focus the search on it.
	if dbg:
		dbg.append( ('q= %s', q) )
	for i in  range(1, len(q)):
		assert q[i][X] != q[i-1][X]
		if (q[i][F]>0) != (q[i-1][F]>0):
			if dbg:
				dbg.append( ('recurse i=%d [%.15g, %.15g] -> [%.15g, %.15g]',
						(i, xl, xh, q[i-1][X], q[i][X]) )
					)
			return _root1(f, q[i-1][X], q[i][X], p, q[i-1][F], q[i][F], epsx, epsf)
	raise RuntimeError, "Lost a root!"



def root(f, xl, xh, p, epsx, epsf):
	"""Find a root of an equation.
	F is the function to solve, called as f(x, p).
	P is arbitrary data for f.
	This function assumes there is known to be a root in [xl,xh].
	It finds it within a region of width epsx, or
	at least finds an x such that -epsf < f(x, p) < epsf .
	"""
	_s = []
	if xh < xl :
		return None
	fl = f(xl, p)
	if fl == 0:
		return xl
	fh = f(xh, p)
	if fh == 0:
		return xh
	if xh == xl:
		return None

	# print fl, fh
	assert (fl>0) != (fh>0), "0 or an even number of roots in initial x-range."
	try:
		return _root1(f, xl, xh, p, fl, fh, epsx, epsf, _s)
	except RuntimeError, x:
		die.warn('Runtime Error: %s' % str(x))
		for (fmt,val) in _s:
			print fmt%val
		die.die('Failed')


def _test1f(x, p):
	return 100/int(round(x)) - 10

def test():
	q = root(_test1f, 1, 1000, None, 1e-6, 0.01)
	assert _test1f(q, None) == 0
	q = root(_test1f, 1, 16, None, 1e-6, 0.01)
	assert _test1f(q, None) == 0


def iroot(y, xl, xh, epsy=0.0):
	"""Find a zero in an array y.
	This function assumes there is known to be a root in [xl,xh].
	It returns a real-number index into the array which
	linearly interpolates to zero.
	"""
	import math
	assert 0 <= xl < y.shape[0]
	assert 0 <= xh < y.shape[0]

	def interp(x, y):
		xi = math.floor(x)
		frac = x - xi
		i = int(xi)
		if frac > 0:
			return y[i] + (y[i+1]-y[i])*frac
		return y[i]

	return root(interp, xl, xh, y, 0.001, epsy)


def testi():
	import Num
	y = 0.64234*(Num.arange(100) - 32.234)
	assert abs(iroot(y, 10, 50) - 32.234) < 0.002

if __name__ == '__main__':
	test()
	testi()




import math
import sys

try:
	from Numeric_gpk import N_maximum, N_minimum, N_median, N_mean_ad,	\
			median_across, N_median_across, variance, stdev,	\
			set_diag,	\
			make_diag, limit, vec_variance, qform,	\
			KolmogorovSmirnov, interp, interpN
except ImportError, _x:
	if str(_x).startswith('cannot import name'):
		raise
	pass


def median(x):
	xx = sorted(x)
	n = len(xx)
	assert n > 0, "No data to median."
	return 0.5*(xx[n//2] + xx[(n-1)//2])


def avg(x):
	s = 0.0
	n = 0
	for t in x:
		s += t
		n += 1
	return s/float(n)


def median_ad(x):
	"""Median absolute deviation"""
	medn = median(x)
	return median( [ abs(t-medn) for t in x ] )
mad = median_ad


def mean_ad(x):
	medn = median(x)
	sum = 0.0
	for t in x:
		sum += abs(t-medn)
	return sum/float(len(x))


def geo_mean(*d):
	"""Geometric mean of its arguments."""
	s = 0.0
	for t in d:
		s += math.log(t)
	return math.exp(s/len(d))



def entropy(x):
	"""Compute the entropy of a list of probabilities."""
	e = 0.0
	ps = 0.0
	for p in x:
		assert p <= 1.0
		if p > 0.0:
			e -= p*math.log(p)
		ps += p
	assert 0.999 < ps < 1.001, "Probabilities must sum to one."
	return e


def resample(d):
	"""Bootstrap resampling.  Call this many times: each one
	returns a random resampling.
	"""
	import random
	o = []
	for i in range(len(d)):
		o.append( random.choice(d) )
	return o


def jackknife(d):
	"""Jackknife resampling.  Call this once.
	It returns a list of deleted lists."""
	for drop in range(len(d)):
		yield [ di for (i, di) in enumerate(d) if i!=drop ]






def Student_t_dens(x, n):
	"""From p.337 Statistical Theory by Bernard Lindgren."""

	from transcendental import gamma
	# http://bonsai.ims.u-tokyo.ac.jp/~mdehoon/software/python/statistics.html	
	p = gamma((n+1)/2.0) * (1+x*x/n)**(-(n+1)/2.0) / (
			math.sqrt(n*math.pi)*gamma(n/2.0)
			)
	return p




_fcache = {}
def log_factorial(n):
	assert n >= 0
	try:
		lf = _fcache[n]
	except KeyError:
		lf = 0.0
		for i in range(2,n):
			lf += math.log(i)
		_fcache[n] = lf
	return lf


def log_Combinations(n, m):
	assert n >= m
	assert m >= 0
	return log_factorial(n) - log_factorial(m) - log_factorial(n-m)

def ComplexMedian(P):
	"""P is a list of complex numbers.
	This algorithm works by repeatedly stripping off the convex
	hull of the points.
	"""
	import convex_hull2d
	import cmath
	import dictops
	HUGE = 1e30
	EPS = 1e-7
	Q = dictops.dict_of_accums()
	for p in P:
		Q.add(p, 1.0)

	while len(Q) > 3:
		# print 'Q=', Q
		edge = convex_hull2d.convexHull(Q.keys())
		# print 'edge=', edge
		ee = (edge[-1],) + edge + (edge[0],)
		wt = {}
		for i in range(1,len(edge)+1):
			em = ee[i] - ee[i-1]
			ep = ee[i+1] - ee[i]
			# angle = cmath.log(em).imag - cmath.log(ep).imag
			if min(abs(ep), abs(em)) < EPS*max(abs(em), abs(ep)):
				angle = math.pi/2.0	#Kluge, mild.   Roundoff errors.
			else:
				angle = cmath.log(em/ep).imag
			# KLUGE AND ROUNDOFF WARNING!
			if angle <= 0.0 and angle > -0.5:
				angle = EPS	#KLUGE!  Awful!
			if angle < 0.0:
				angle += 2*math.pi
			if angle >= math.pi and angle<math.pi+0.5:
				angle = math.pi-EPS	# KLUGE!  Awful!
			# print 'angle=', angle, ee[i-1], ee[i], ee[i+1], 'pi=', math.pi
			assert angle>=0 and angle<=math.pi, "angle=%g" % angle
			wt[ee[i]] = angle
		fmin = HUGE
		for p in edge:
			f = Q[p]/wt[p]
			if f < fmin:
				fmin = f
		# print 'fmin=', fmin
		assert fmin > 0.0
		sum = complex()
		swt = 0.0
		for p in edge:
			fwp = fmin * wt[p]
			# print 'Subtracting wt of ', fwp, 'from Q[', p, ']=', Q[p]
			swt  += fwp
			sum += p * fwp
			if Q[p] > fwp+EPS:
				Q[p] -= fwp
				# print '\tQ[', p, ']=', Q[p]
			else:	# Q[p]<=fwp
				assert abs(Q[p]-fwp) < EPS
				# print 'Deleting Q[', p, ']=', Q[p]
				del Q[p]
		if len(Q) == 0:
			return sum/swt
	# print 'Qfinal=', Q
	sum = complex()
	w = 0.0
	for (p,n) in Q.items():
		sum += p*n
		w += n
	return sum/w



def testCM():
	def eq(a, b):
		tmp = abs(a-b)/(abs(a)+abs(b)) < 1e-6
		if not tmp:
			print 'eq fails: %s vs %s' % (str(a), str(b))
		return tmp
	print 'CM'
	assert eq(ComplexMedian([complex(1,0), complex(2,0), complex(3,0)]), 2)
	assert eq(ComplexMedian([complex(1), complex(2), complex(3), complex(4)]), 2.5)
	assert eq(ComplexMedian([complex(1), complex(2), complex(3), complex(4), complex(5)]), 3)
	assert eq(ComplexMedian([complex(1), complex(2), complex(3), complex(4), complex(4)]), 3)
	assert eq(ComplexMedian([complex(1,1), complex(2,2), complex(3,3), complex(4,4)]),
			complex(2.5,2.5))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1), complex(1,1)]),
			complex(0.5,0.5))
	assert eq( ComplexMedian([complex(0,0), complex(1,0), complex(0,1),
				complex(1,1), complex(1,1)]),
			complex(1,1))
	assert eq( ComplexMedian([complex(0,0), complex(1,0), complex(0,1),
				complex(1,1), complex(0.6,0.6)]),
			complex(0.6,0.6))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1), complex(1,1),
				complex(0,0), complex(2,0), complex(0,2), complex(2,2)]),
			complex(0.5,0.5))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1)]),
			complex(1./3., 1./3.))
	assert eq(ComplexMedian([complex(0,0), complex(1,0), complex(0,1),
				complex(0,0), complex(2,0), complex(0,2)]),
			complex(0.3, 0.3))
	import cmath
	for N in [3, 4, 5, 6, 7, 8, 13, 40, 100, 2351]:
		print 'N=', N
		assert eq(ComplexMedian( [ 1+cmath.exp(2*cmath.pi*1j*float(q)/N)
						for q in range(N) ]),
				complex(1.0, 0.0)
				)





import Queue
import threading
class threaded_readable_file(object):
	QSIZE = 100
	_string = type('')

	def __init__(self, fd):
		self.q = Queue.Queue(self.QSIZE)

		def rhelper(fd, q):
			try:
				for l in fd:
					q.put(l)
				q.put(None)
				# while True:
					# l = fd.readline()
					# if l == '':
						# q.put(None)
						# break
					# else:
						# q.put(l)
						
			except (Exception, KeyboardInterrupt):
				q.put( sys.exc_info() )
	
		self.rh = threading.Thread(target=rhelper, args=(fd, self.q))
		self.rh.start()


	def readline(self):
		if self.q is None:
			return ''
		x = self.q.get()
		if type(x) != self._string:
			self.rh.join()
			self.q = None
			if x is not None:
				raise x[0], x[1], x[2]
			return ''
		return x


	def readlines(self):
		o = []
		while self.q is not None:
			x = self.q.get()
			if type(x) != self._string:
				self.rh.join()
				self.q = None
				if x is not None:
					raise x[0], x[1], x[2]
				break
			o.append( x )
		return o


	def read_iter(self):
		while self.q is not None:
			x = self.q.get()
			if type(x) != self._string:
				self.rh.join()
				self.q = None
				if x is not None:
					raise x[0], x[1], x[2]
				break
			yield x

	__iter__ = read_iter


def thr_iter_read(fd):
	"""Read the contents of a file as an iterator.
	The read is two-threaded, so that one thread can be
	waiting on disk I/O while the other thread is
	processing the results.
	"""
	x = threaded_readable_file(fd)
	return x.read_iter()



def makedirs(fname, mode=0775):
	"""This makes the specified directory, including all
	necessary directories above it.    It is like os.makedirs(),
	except that if the directory already exists, it
	does not raise an exception.
	"""
	import os
	try:
		os.makedirs(fname, mode)
	except OSError, x:
		if os.path.isdir(fname) and os.access(fname, os.F_OK):
			return
		raise



def shuffle_Nrep(y, n=1, cmp=None):
	"""Shuffle a list, y,
	so that no item occurs more than n times in a row.
	Equality is determined by the comparison function cmp returning zero.
	"""
	import random
	assert n>0, "Silly!"
	x = y[:]
	random.shuffle(x)
	m = len(x)
	if cmp is None:
		cmp = lambda a, b: a==b

	passes = 0
	restart = 0
	while passes<1000:
		prb = None
		pstart = 0
		reps = 0
		# print 'x=', x
		for i in range(max(1, restart), m):
			if cmp(x[i-1], x[i]):
				reps += 1
				pstart = i - 1
			else:
				pstart = None
				reps = 0
			if reps >= n:
				prb = i
				break
		if prb is None:
			y[:] = x
			return y

		k = prb+1
		found = False
		while k < m:
			if not cmp(x[pstart], x[k]):
				found = True
				break
			k += 1
		if not found:
			# print 'ROLL'
			tmp = x.pop(0)
			x.append(tmp)
			restart = 0
			continue
		a = pstart
		b = k + 1
		# print 'a,b=', a, b
		tmp = x[a:b]
		random.shuffle(tmp)
		# print 'tmp=', x[a:b], '->', tmp
		x[a:b] = tmp
		restart = a
		passes += 1
	raise RuntimeError, 'Too many passes: cannot avoid repetitions'


def testSNR():
	import random
	for i in range(20):
		x = [ i//10 for i in range(20) ]
		shuffle_Nrep(x, 1)
		for i in range(len(x)-1):
			assert x[i] != x[i+1], 'Whoops: i=%d, %d %d' % (i, x[i], x[i+1])
		x = [ i//3 for i in range(10000) ]
		random.shuffle(x)
		shuffle_Nrep(x, 1)
		for i in range(len(x)-1):
			assert x[i] != x[i+1], 'Whoops: i=%d, %d %d' % (i, x[i], x[i+1])


def open_nowipe(nm1, nm2, mode="w"):
	"""Open a file (typically for writing) but make sure that the
	file doesn't already exist.   The name is constructed from
	nm1 a sequence number and nm2.   The sequence number gets incremented
	until a name is found that doesn't exist.
	Typically, nm2 is a suffix like ".wav"
	@param nm1: the part of the name to the left of the sequence number
	@param nm2: the part of the name to the right of the sequence number
	@param mode: The way to open the file -- passed to open().
	@type nm1: str
	@type nm2: str
	@type mode: str
	"""
	i = 0
	try:
		while True:
			nm = '%s%d%s' % (nm1, i, nm2)
			open(nm, 'r')
			i += 1
	except IOError:
		pass
	return (open(nm, mode), nm)


def dropfront(prefix, s):
	"""Drops the prefix from the string and raises an exception
	if it is not there to be dropped.
	"""
	if not s.startswith(prefix):
		raise ValueError, "String '%s' must start with '%s'" % (s[:20], prefix)
	return s[len(prefix):]


def open_compressed(fn):
	import g_pipe

	if fn.endswith('.bz2'):
		ci, co = g_pipe.popen2("bzcat", ['bzcat', fn])
		ci.close()
		return co
	elif fn.endswith('.gz'):
		ci, co = g_pipe.popen2("zcat", ['zcat', fn])
		ci.close()
		return co
	return open(fn, 'r')


def gammaln(x):
    """
Returns the gamma function of xx.
Gamma(z) = Integral(0,infinity) of t^(z-1)exp(-t) dt.
(Adapted from: Numerical Recipies in C. via code
Copyright (c) 1999-2000 Gary Strangman and released under the LGPL.)
"""
    coeff = [76.18009173, -86.50532033, 24.01409822, -1.231739516,
             0.120858003e-2, -0.536382e-5]
    tmp = x + 4.5
    tmp -= (x-0.5)*math.log(tmp)
    ser = 1.0
    for cj in coeff:
        ser += cj/x
        x += 1
    return -tmp + math.log(2.50662827465*ser)


def log_t_density(x, nu):
	c = gammaln((nu+1)/2.0) - 0.5*math.log(nu*math.pi) - gammaln(nu/2.0)
	return math.log(1+x**2/nu)*(-(nu+1)/2.0)



_a_factor_cache = {1:1, 2:2, 3:3}	# THIS CACHE SHOULD BE LOCKED
def a_factor(n):	# Don't mess with private_start.
	"""Finds the smallest prime factor of a number."""

	try:
		return _a_factor_cache[n]
	except KeyError:
		pass
	for p in primes():
		if p*p > n:
			f = n
			break
		if n%p == 0:
			f = p
			break
	_a_factor_cache[n] = f
	return f


_primes = [2, 3]	# 2, 3 *must* be there.	# THIS CACHE SHOULD BE LOCKED
def primes():
	"""This is a generator that produces an infinite list of primes.
	"""
	for p in _primes:
		yield p
	i = _primes[-1] + 2
	while True:
		for p in primes():
			if p*p > i:
				if p > _primes[-1]:	# multithreading...
					_primes.append(i)
				yield i
			if i%p == 0:
				i += 2
				break


_factor_cache = {}	# THIS CACHE SHOULD BE LOCKED
def factor(n):	# Don't mess with private_start.
	"""Factor a number into a list of prime factors,
	in increasing order.
	@param n: input number
	@type n: int
	@return: prime factors
	@rtype: list
	"""

	try:
		return _factor_cache[n]
	except KeyError:
		pass
	f = a_factor(n)
	if f == n:
		tmp = [n]
	else:
		tmp = [f] + factor(n//f)
	_factor_cache[n] = tmp[:]
	return tmp


def test_primes():
	assert factor(11) == [11]
	assert factor(16) == [2, 2, 2, 2]
	assert factor(100) == [2, 2, 5, 5]
	assert factor(2) == [2]
	assert factor(97) == [97]
	assert factor(55) == [5, 11]
	assert gcd(5,3)==1
	assert gcd(14, 21)==7
	assert gcd(100, 70)==10

def gcd(a, b):
	"""Greatest common factor/denominator.
	@type a: int
	@type b: int
	@rtype: int
	@return: the greatest common factor of a and b.
	"""
	assert a >= 0 and b >= 0
	while b != 0:
		tmp = b
		b = a % b
		a = tmp
	return a


if __name__ == '__main__':
	testCM()
	testSNR()
	test_primes()



import lapack_dge
import Numeric


def _lg(n):
	k = 0
	twok = 1
	while twok < n:
		k += 1
		twok *= 2
	return k

def dgesv(a, b):
	"""Solve a*x = b."""
	a = Numeric.array(a, Numeric.Float)
	b = Numeric.array(b, Numeric.Float)
	n = a.shape[0]
	assert a.shape == (n, n)
	nrhs = b.shape[1]
	assert b.shape == (n, nrhs)
	ipiv = Numeric.zeros((n,), Numeric.Int)
	info = 0
	o = lapack_dge.dgesv(n, nrhs, a, n, ipiv, b, n, info)
	assert o['info'] == 0
	return b



if __name__ == '__main__':
	a = Numeric.array([[2, 0], [0, 1]], Numeric.Float)
	b = Numeric.array([[1],[1]], Numeric.Float)
	print "a=", a
	print "b=", b
	print dgesv(a, b)



"""A class that implements a simple file in memory.
It's not an exact simulation of a real file:
it assumes that no seeking is done, and that
all reads are line-at-a-time.
It also (intentionally) allow reading from files opend
in mode 'r', and allows writing for files opened in mode
'w'."""

import types

__version__ = "$Revision: 1.4 $"


class file:
	def __init__(self, name, mode):
		assert mode in ['r', 'w',  'rw',  'wr'], "Bad mode: %s" % mode
		self.name = name
		self.mode = mode
		self.buf = ['']
		self.p = 0

	def write(self, data):
		if len(data) == 0:
			return 0
		self.buf[-1] = self.buf[-1] + data
		if data[-1] == '\n':
			self.buf.append('')
		return len(data)

	def writelines(self, a):
		if type(a) == types.StringType:
			self.write(a)
			return
		for t in a:
			self.write(t)

	def readline(self):
		if self.p >= len(self.buf):
			return ''
		tmp = self.buf[self.p]
		self.p += 1
		return tmp


	def readlines(self):
		if self.p >= len(self.buf):
			return []
		tmp = self.buf[self.p:]
		if tmp[-1] == '':
			tmp = tmp[:-1]
			self.p = len(self.buf) - 1
		else:
			self.p = len(self.buf)
		return tmp


	def flush(self):
		pass


	def close(self):
		pass



def open(name, mode):
	return file(name, mode)



if __name__ == '__main__':
	x = file("foo", "rw")
	x.write("Hello\n")
	x.write("foo\n")
	assert x.readline() == "Hello\n"
	assert x.readline() == "foo\n"
	assert x.readline() == ''
	assert x.readlines() == []

	x = file("foo", "rw")
	x.write("Hello\n")
	x.write("foo\n")
	assert x.readlines() == [ 'Hello\n', 'foo\n' ]



"""Do mathematical operations on time series, where
the two operands don't necessarily have the same
sampling times.     It finds a common sampling time
and sampling interval, then interpolates as necessary
to bring the data onto the common time axis.
"""

import Num
import gpkimgclass
import math
import gpkmisc


class axis:
	"""This is a time axis for a time series.
	Indices of the underlying array are assumed to be zero-based.
	"""

	def __init__(self, start=None, dt=None, n=None, end=None, crpix=0):
		if start is not None and end is not None and n is not None:
			self.crval = start
			self.n = int(round(n))
			self.cdelt = (end-start)/float(n-1)
		elif start is not None and n is not None and dt is not None:
			self.crval = start
			self.n = n
			self.cdelt = dt
		elif start is not None and dt is not None and end is not None:
			self.crval = start
			self.n = 1 + abs(int(round((end-start)/dt)))
			# print '# self.n = 1+round(', (end-start)/dt, ')=', self.n
			if end > start:
				self.cdelt = dt
			else:
				self.cdelt = -dt
		else:
			raise ValueError, "Either silly values or not implemented."
		assert self.n >= 0, "Silly number of data"
		self.crpix = crpix
		# print 'info=', self.crval, 'at', self.crpix, 'dt=', self.cdelt, 'n=', self.n
		assert round(self.index(self.start())) == 0, 'start fail: %s' % self
		assert round(self.index(self.end())) == self.n-1, 'end fail: %s' % self

	def N(self):
		return self.n

	def coord(self, index):
		"""What is the i^th coordinate if the i^th index==index?
		More plainly, this function gives you the coordinate that
		corresponds to the index."""
		assert self.n >= 0
		return self.crval + (index-self.crpix) * self.cdelt
	

	def coords(self):
		"""Generate an array of all the time values."""
		assert self.n >= 0
		index = Num.arrayrange(self.n)
		return index*self.cdelt + (self.crval-self.crpix*self.cdelt)


	def index(self, t):
		assert self.n > 0
		assert self.cdelt != 0
		tmp = self.crpix + (t-self.crval)/self.cdelt
		if not 0 <= round(tmp) < self.n:
			raise IndexError, "coordinate out of range: %g->%.1f" % (t, tmp)
		return tmp


	def indices(self, t):
		assert self.n > 0
		assert self.cdelt != 0
		tmp = self.crpix + (t-self.crval)/self.cdelt
		chk = Num.less(tmp, -0.5) + Num.greater_equal(tmp, self.n-0.5)
		if Num.sometrue(chk):
			nbad = Num.sum(Num.greater(chk, 0))
			raise IndexError, "%d coordinates out of range" % nbad
		return tmp


	def dt(self):
		return self.cdelt


	def start(self):
		return self.coord(0)

	def end(self):
		return self.coord(self.n-1)

	def __str__(self):
		return '<axis start=%g end=%g dt=%g n=%d>' % (self.start(), self.end(),
								self.dt(), self.N()
								)
	__repr__ = __str__


def time(datasets, start=None, end=None):
	"""Computes a common time axis for several datasets."""
	dts = 0.0
	dtn = 0
	for x in datasets:
		tdt = x.dt()
		dts += math.log(tdt)
		dtn += 1
	dt = math.exp(dts/dtn)

	_s = [x.start() for x in datasets ]
	if start is not None:
		_s.append(start)
	_start = max(_s)
	_e = [x.end() for x in datasets]
	if end is not None:
		_e.append(end)
	_end = min(_e)

	n = int(math.floor( ( _end - _start ) / dt ) )
	if n < 0:
		n = 0
	return axis( start=_start, dt=dt, n=n )


def time2(a, b):
	dt = 0.5*( a.dt() + b.dt() )
	start = max(a.start(), b.start())
	end0 = min(a.end(), b.end())
	n = int(math.floor((end0-start)/dt))
	return axis(start=start, dt=dt, n=n)


def _fill_interp_guts(a, t, fill, interpolator):
	"""Returns a Numeric array."""
	assert len(t.shape) == 1
	if a.n[2] == 0:
		# This case catches empty datasets.
		return Num.zeros((t.shape[0], a.n[1]), Num.Float) + fill
	noneedfill = Num.greater_equal(t, a.start()) * Num.less_equal(t, a.end())
	assert len(noneedfill.shape) == 1
	# print 'nnf', noneedfill
	nofillidx = Num.nonzero(noneedfill)[0]
	# print 'nfi', nofillidx
	if nofillidx.shape != t.shape:
		# print 't.shape=', t.shape
		ainofill = interpolator(a, Num.take(t, nofillidx, axis=0))
		# print 'ainf', ainofill
		out = Num.zeros((t.shape[0], a.n[1]), Num.Float) + fill
		# print 'out1=', out
		Num.put(out, nofillidx, ainofill)
		# print 'out2=', out
		# print 'outshape=', out.shape
	else:
		out = interpolator(a, t)
		# print 'Outshape=', out.shape
	return out


def test_fig():
	print "TEST_FIG"
	a = axis(start=2.0, dt=1.0, n=10)
	a.n = (0, 1, 10)
	t = Num.arrayrange(20)
	fill = -1
	ifcn = lambda a, b: 100
	q = _fill_interp_guts(a, t, fill, ifcn)
	print q
	print q.shape


def interp_fill(a, t, fill):
	return _fill_interp_guts(a, t, fill, interp)

def interpN_fill(a, t, fill):
	return _fill_interp_guts(a, t, fill, interpN)



def interp(a, t):
	"""Interpolate to a specified time axis.
	This does a linear interpolation.
	A is a gpkimgclass, and t is an array of times.
	Returns a Numeric array.
	"""
	idx = a.t_index(t)
	return gpkmisc.interp(a.d, idx)



def test_interp1():
	x = Num.array([0, 1], Num.Float)
	xx = gpkimgclass.gpk_img({'CDELT2':1.0, 'CRPIX2':1, 'CRVAL2':0}, x)
	t = Num.array([0.0, 0.7, 0.99, 1.0], Num.Float)
	q = interp(xx, t)
	print q.shape, t.shape, x.shape, xx.d.shape
	print q
	assert q.shape[0] == t.shape[0]
	assert len(q.shape) == len(xx.d.shape)
	assert Num.sum(Num.absolute(q-[[0.0], [0.7], [0.99], [1.0]]))<1e-6
	qq = gpkimgclass.gpk_img({'CDELT2':1.0, 'CRPIX2':1, 'CRVAL2':0}, q)
	print 'qq.n', qq.n, xx.n
	assert qq.n[1] == xx.n[1] and qq.n[2] == t.shape[0]
	qn = interpN(xx, t)
	print 'qn=', qn
	assert Num.sum(Num.absolute(qn-[[0.0], [1.0], [1.0], [1.0]]))<1e-6

def test_interp2():
	x = Num.array([0, 1, 1.5], Num.Float)
	xx = gpkimgclass.gpk_img({'CDELT2':1.0, 'CRPIX2':1, 'CRVAL2':0}, x)
	t = Num.array([0.0, 0.7, 0.99, 1.0, 1.1, 2.0], Num.Float)
	q = interp(xx, t)
	assert Num.sum(Num.absolute(q-[[0.0], [0.7], [0.99], [1.0], [1.05], [1.5]]))<1e-6
	qn = interp(xx, t)
	assert Num.sum(Num.absolute(qn-[[0.0], [0.7], [0.99], [1.0], [1.05], [1.5]]))<1e-6


def test_interp3():
	x = Num.array([[0, 100], [1, 101], [1.5, 101.5]], Num.Float)
	xx = gpkimgclass.gpk_img({'CDELT2':1.0, 'CRPIX2':1, 'CRVAL2':0}, x)
	t = Num.array([0.0, 0.7, 0.99, 1.0, 1.1, 2.0], Num.Float)
	q = interp(xx, t)
	print q
	qq = gpkimgclass.gpk_img({'CDELT2':1.0, 'CRPIX2':1, 'CRVAL2':0}, q)
	print 'qq.n', qq.n, xx.n
	assert qq.n[1] == xx.n[1] and qq.n[2] == t.shape[0]
	assert Num.sum(Num.absolute(q-[[0.0, 100.0], [0.7, 100.7], [0.99, 100.99],
					[1.0, 101.0], [1.05, 101.05], [1.5, 101.5]]))<1e-6
	qn = interp(xx, t)
	assert Num.sum(Num.absolute(qn-[[0.0, 100.0], [0.7, 100.7], [0.99, 100.99],
					[1.0, 101.0], [1.05, 101.05], [1.5, 101.5]]))<1e-6



def interpN(a, t):
	"""Interpolate to a specified time axis via
	nearest-neighbor interpolation.
	A is a gpkimgclass, and t is an array of times.
	Returns a Numeric array, not a gpkimgclass.
	"""
	# print "interp.a=", a
	# print "interp.t=", t
	idx = a.t_index(t)
	return gpkmisc.interpN(a.d, idx)



def common(data_sets, start=None, end=None):
	"""Put several datasets on a common time axis.
	Linearly interpolate as needed.
	Data_sets = [ class gpkimgclass.gpk_img  or equivalent],
	the function returns [ time, array1, array2,...]
	where arrayN is a 2-dimensional Numeric array
	for each dataset.
	"""
	tt = time(data_sets, start=start, end=end)
	t = tt.coords()

	return [tt] + [ interp(x, t) for x in data_sets ]



def commonN(data_sets, start=None, end=None):
	"""Put several data sets on a common time axis.
	Interpolate by choosing nearest neighbor.
	"""
	tt = time(data_sets, start=start, end=end)
	t = tt.coords()

	return [t] + [ interpN(x, t) for x in data_sets ]


def mul(a, b, hdr_op=None):
	tt = time((a, b))
	t = tt.coords()
	# print "t=", t
	ai = interp(a, t)
	bi = interp(b, t)
	# print "ai=", ai
	# print "bi=", bi
	c = ai * bi
	# print "c=", c

	if hdr_op is None:
		h = {}
	else:
		h = hdr_op(a.hdr, b.hdr)

	h['CDELT2'] = tt.dt()
	h['CRVAL2'] = tt.start()
	h['CRPIX2'] = 1
	
	return gpkimgclass.gpk_img(h, c)



def copy_interval(a, t0, t1, hdr_op=None, mode="rr"):
	"""This copies the part of the time-series in a
	where t0 < t < t1.
	'a' is a gpk_img object.
	"""

	if hdr_op is None:
		h = a.hdr.copy()
	else:
		h = hdr_op(a.hdr, t0, t1)
	i0 = a.t_index(t0)
	i1 = a.t_index(t1)
	if mode[0] == "r":
		i0 = int(round(i0))
	elif mode[0] == "w":
		i0 = int(math.floor(i0))
	elif mode[0] == "n":
		i0 = int(math.ceil(i0))
	if mode[1] == "r":
		i1 = int(round(i1))
	elif mode[1] == "w":
		i1 = int(math.floor(i1))
	elif mode[1] == "n":
		i1 = int(math.ceil(i1))

	d = Num.array(a.d[i0:i1,:], Num.Float, copy=True)
	h['CRVAL2'] = a.time(i0)
	h['CRPIX2'] = 1
	return gpkimgclass.gpk_img(h, d)


def apply(fcn, a, hdrfcn=lambda x:x):
	"""Apply a function, point-by-point to the data in a."""
	return gpkimgclass.gpk_img(hdrfcn(a.hdr), fcn(a.d))


def resample(a, dt):
	import samplerate
	ratio = a.dt()/dt
	d = samplerate.resample(a.d, ratio)
	assert len(d.shape)==len(a.d.shape)
	assert d.shape[1]==a.d.shape[1], "d.shape[0]: %d -> %d" % (a.d.shape[0], d.shape[0])
	assert abs(float(d.shape[0])/float(a.d.shape[0]) - ratio) < 0.1
	h = a.hdr.copy()
	h['CDELT2'] = dt
	return gpkimgclass.gpk_img(h, d)


def test():
	a = gpkimgclass.gpk_img({'CDELT2':1.0, 'CRPIX2':1, 'CRVAL2':0.0},
				Num.arrayrange(10))
	b = gpkimgclass.gpk_img({'CDELT2':0.5, 'CRPIX2':1, 'CRVAL2':0.4},
				Num.arrayrange(10)*0.5+0.4)
	# print 'a=', a.d
	# print 'b=', b.d
	# print 'b.time()', b.time()
	ab = mul(a, b)
	# print 'ab=', ab.d
	# print 'ab.time()=', ab.time()
	err = ab.d - Num.transpose([ab.time()])**2
	# print 'err=', err
	assert Num.sum(err**2) < 1e-5



if __name__ == '__main__':
	test_interp1()
	test_interp2()
	test_interp3()
	test()



import Num
import types

import sbd_array

class BadParamError(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)

class NoTermsEx(ValueError):
	def __init__(self, s='Empty Equation'):
		ValueError.__init__(self, s)

class eqn_c:
	def index_ends(self):
		if len(self.terms) == 0:
			raise NoTermsEx
		return (self.terms[0][0], self.terms[-1][0])

	def __init__(self, terms, sum, wt, index_ctr):
		"""Takes an equation in the form:
		[(index, coefficient), ...] = sum
		and stores it away into a class.
		"""
		if type(terms) != types.ListType:
			raise TypeError, 'first arg to eqn_c is a list'
		if type(index_ctr)!=types.IntType:
			raise TypeError, 'index_ctr in eqn_c is a int'
		if wt < 0:
			raise BadParamError, 'wt<0'
	
		# for t in terms:
			# assert type(t[0])==types.IntType, 'Not int index: T=%s'%str(t)
		self.terms = map(lambda x, w=wt, c=index_ctr: (x[0]+c, x[1]*w),
					terms)
		self.terms.sort()	# Tuples are sorted by their first element first.
		self.sum = sum*wt

	def __str__(self):
		termlist = map(lambda x: "%g*x[%d] "%(x[1], x[0]), self.terms)
		return '<eqn_c: ' + ' '.join(termlist) + " = %g >"%self.sum

	def substitute(self, known, zero=0):
		"""Modifies an equation by substituting in known values.
		Argument 'known' is an array of values."""
		try:
			sis, sie = self.index_ends()
		except NoTermsEx:
			return
		if sis >= zero:
			return
		o = []
		for t in self.terms:
			if t[0] >= zero :
				o.append(t)
			else:
				# print "substitute: known[", t[0], "]=", known[t[0]-zero]
				try:
					self.sum = self.sum - t[1]*known[t[0]-zero]
				except IndexError:
					raise RuntimeError, "Sorry. Previous phrase too short or smooth too big."
				# Here we make use of the fact that known[-1] == the last value
		self.terms = o
				
	def __nonzero__(self):
		return len(self.terms)>0



def range_of_indices(l):
	"""This finds the highest and lowest indices for a list of equations."""
	xmin = []
	xmax = []
	for q in l:
		try:
			t = q.index_ends()
		except NoTermsEx:
			continue
		xmax.append(t[1])
		xmin.append(t[0])
	return (min(xmin), max(xmax))


def bandwidth(l):
	bw = 0
	for q in l:
		try:
			ts, te = q.index_ends()
		except NoTermsEx:
			continue
		tmp = te - ts
		if tmp > bw:
			bw = tmp
	return bw

def build_matrices(l, n=None, maxbw=None):
	"""Takes a list of eqn_c classes, and builds the
	matrices to be solved.
	"""
	m = len(l)
	if n is None:
		t = range_of_indices(l)
		assert t[0] == 0
		n = t[1] + 1
	b = Num.zeros((n,), Num.Float)

	if maxbw is None:
		maxbw = bandwidth(l)
	a = sbd_array.sbd(n, maxbw)

	for e in l:
		try:
			ts, te = e.index_ends()
		except NoTermsEx:
			continue
		assert ts >= 0 and te < n
		tt = Num.zeros((te-ts+1,), Num.Float)
		for (t10, t11) in e.terms:
			idx = t10 - ts
			tt[idx] = tt[idx] + t11
			b[t10] = b[t10] + t11 * e.sum
		block = Num.multiply.outer(tt, tt)
		a.bd_increment(ts, block)
	return (a, b)



def test_eqn():
	e0 = []
	e0.append(eqn_c([(0,1)], 3.0, 2.0, 0))
	a, b = build_matrices(e0)
	# print "a=", a
	# print "b=", b
	assert a.shape == (1, 1)
	assert a[0,0] == 4.0
	assert b[0] == 12.0


def add_something(a, thing, scale):
	"""Add a band-diagonal part to matrix a.
	We assume a has shared-reference semantics, like
	the Numeric module.  This will *not* work for sbd_array,
	which has copy semantics for __getslice__.
	"""
	nt = len(thing)
	assert a.shape[0] == a.shape[1]
	assert a.shape[0] > 2*nt+1
	c = map(lambda x, sc=scale: Num.asarray(x)*sc, thing)
	ash = a.shape
	for i in range(nt-1):
		assert len(c[i].shape)==1
		aslice = a[i, 0:c[i].shape[0]]
		Num.add(aslice, c[i], aslice)
		eslice = a[-1-i, -1:-c[i].shape[0]-1:-1]
		Num.add(eslice, c[i], eslice)

	assert c[-1].shape[0]%2 == 1
	hmm = c[-1].shape[0]/2
	assert hmm == nt-1

	cnt = c[nt-1]
	for i in range(hmm, ash[0]-hmm):
		aslice = a[i,i-hmm:i+hmm+1]
		Num.add(aslice, cnt, aslice)

	return 1 + hmm


def add_slope_curv_droop(a, sls, cs, droop):
	"""Adds a slope and curvature part to the matrix.
	Returns the bandwidth of the part it added.
	"""
	add_something(a, [[1]], droop)
	add_something(a, [[1, -1], [-1, 2, -1]], sls)
	add_something(a, [[-1, 2, -1],
				[-2, 5, -4, 1],
				[1, -4, 6, -4, 1]], cs)




def full_matrices(l, n, sls, cs, droop):
	a, b = build_matrices(l, n)
	add_slope_curv_droop(a, sls, cs, droop)
	return (a, b)



def solve_eqns(a, b):
	return sbd_array.solve(a, b)




def test_solve():
	e = []
	e.append(eqn_c([(0,2)], 1.0, 1.0, 0))
	e.append(eqn_c([(0,1)], 2.0, 1.0, 1))
	a, b = build_matrices(e)
	# print "a=", a
	# print "b=", b
	x = solve_eqns(a, b)
	# print "x=", x
	assert x[0,0] == 0.5
	assert x[0,1] == 2.0
	e = []
	e.append(eqn_c([(0,1), (1,1)], 1.0, 1.0, 0))
	e.append(eqn_c([(0,-1), (1,1)], 1.0, 1.0, 0))
	a, b = build_matrices(e)
	x = solve_eqns(a, b)
	# print "x=", x
	assert abs(x[0,0]) < 1e-6
	assert abs(x[0,1] - 1) < 1e-6
	e = []
	e.append(eqn_c([(0,1)], 1.0, 2.0, 0))
	e.append(eqn_c([(0,1)], 2.0, 1.0, 0))
	a, b = build_matrices(e)
	x = solve_eqns(a, b)
	# print "a=", a, "b=", b, "x=", x
	assert abs(x[0,0] - 6./5.) < 1e-6

def test_solve2():
	e = []
	e.append(eqn_c([(0,-1), (1,1)], 1.0, 1.0, 0))
	e.append(eqn_c([(0,-1), (1,1)], 1.0, 1.0, 1))
	e.append(eqn_c([(0,-1), (1,1)], 1.0, 1.0, 2))
	e.append(eqn_c([(0,1)], 1.0, 1.0, 0))
	# e.append(eqn_c([(0,1)], 1.0, 1.0, 0))
	a, b = build_matrices(e)
	# print "a=", a
	# print "b=", b
	x = solve_eqns(a, b)
	# print "x=", x
	assert x.shape == (1, 4)
	assert abs(x[0,0] - 1) < 1e-6
	assert abs(x[0,1] - 2) < 1e-6
	assert abs(x[0,2] - 3) < 1e-6
	assert abs(x[0,3] - 4) < 1e-6

def test_bw():
	e = []
	e.append(eqn_c([], 0.0, 1.0, 0))
	assert bandwidth(e) == 0
	e.append(eqn_c([(0,-1), (1,1)], 1.0, 1.0, 0))
	assert bandwidth(e) == 1



def test_as():
	"""Won't work for sbd_array."""
	a = Num.zeros((10,10), Num.Float)
	add_something(a, [[1, -1], [2, -4, 2]], 1.0)
	# print a
	assert a[0,0] == 1
	assert a[0,1] == -1
	assert a[-1,-1] == 1
	for i in range(2, 8):
		assert a[i,i] == -4
		assert a[i-1, i] == 2
		assert a[i+1, i] == 2
		assert a[i, i-1] == 2
		assert a[i, i+1] == 2



def test():
	test_as()
	test_eqn()
	test_solve()
	test_solve2()
	test_bw()


if __name__ == '__main__' :
	test()
	# print "OK: passed tests"



"""Linear prediction of signals"""

import Numeric
import lapack_dgesv




def lp1_fit(y, x, indices, noise=0.0):
	"""Predicts y[i] in terms of the set of x[i+q] for q in indices."""
	y = Numeric.asarray(y, Numeric.Float)
	x = Numeric.asarray(x, Numeric.Float)
	m = x.shape[0]
	assert x.shape == (m,)
	assert y.shape == (m,)
	idx = indices[:]
	idx.sort()
	nix = len(idx)
	imin = idx[0]
	imax = idx[-1]
	noise2 = noise*noise
	n = len(indices)
	phi = Numeric.zeros((n,n), Numeric.Float)
	phistar = Numeric.zeros((n,1), Numeric.Float)
	s = max(-imin, 0)
	e = min(m - imax - 1, m-1)
	mm = e - s + 1
	fmm = float(mm)
	for i in range(nix):
		ii = idx[i]
		for j in range(nix):
			jj = idx[j]
			phi[i,j] = Numeric.dot(x[s+ii:s+ii+mm], x[s+jj:s+jj+mm])/fmm
		phi[i,i] += noise2
		phistar[i,0] += Numeric.dot(y[s:s+mm], x[s+ii:s+ii+mm])/fmm
	# print "phistar=", phistar
	# print "phi=", phi
	coef = lapack_dgesv.dgesv(phi, phistar)
	return (coef[:,0], idx)



def add_sloppy(a, b, offset):
	"""Accumulate b onto a.
	Allow b to be smaller (fill with zeros), or larger (drop)."""
	n = a.shape[0]
	m = b.shape[0]
	if offset < 0:
		b0 = -offset
		a0 = 0
	else:
		b0 = 0
		a0 = offset
	ae = n
	be = m
	if ae-a0 > be-b0:
		ae = a0 + (be-b0)
	if be-b0 > ae-a0:
		be = b0 + (ae-a0)
	# print "a=", a0, ae, "b=", b0, be
	# print "a.shape=", a.shape, "b.shape=", b.shape
	Numeric.add(a[a0:ae], b[b0:be], a[a0:ae])



def lp1_run(x, coef, indices):
	idx = indices[:]
	idx.sort()
	x = Numeric.asarray(x, Numeric.Float)
	n = x.shape[0]
	o = Numeric.zeros((n,), Numeric.Float)
	for j in range(len(idx)):
		add_sloppy(o, x*coef[j], -idx[j])
	return o



if __name__ == '__main__':
	di = [0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0]
	d = [0, 0.5, 1, 0.5, 0.5, 1, 0.5, 0.5, 1, 0.5, 0.5, 1, 0.5,
		0, 0.5, 1, 0.5, 0, 0, 0.5, 1, 0.5, 0, 0]
	
	coef, idx = lp1_fit(d, di, [-1, 0, 1], 0.0001)
	tmp = lp1_run(di, coef, idx)
	assert Numeric.absolute(Numeric.ravel(tmp-d)) < 0.0001

	coef, idx = lp1_fit(di[1:], di[:-1], [-1, 0, 1], 0.0001)
	tmp = lp1_run(di[:-1], coef, idx)
	assert Numeric.absolute(Numeric.ravel(tmp-di[1:])) < 0.0001



"""Resample a time series by a factor r.  I.e. sample more
densely (r<1) or less densely (r>1).
Also see tsops.py."""


import Num


def simple(a, r, direction=0):
	aa = Num.array(a)
	newlen = 1 + int((aa.shape[direction]-1)/r)
	t = Num.around(Num.arrayrange(newlen)*r)
	# print "t=", t
	aswap = Num.swapaxes(aa, direction, 0)
	interped = Num.choose(t, aswap)
	return Num.swapaxes(interped, direction, 0)


def compare(a, b):
	assert len(a) == len(b)
	for i in range(len(a)):
		assert(a[i] == b[i])


def test():
	x = simple([0,1,2,3,4,5,6,7,8,9], 2.0)
	compare(x, [0, 2, 4, 6, 8])
	x = simple([0, 1, 2, 3, 4], 0.5)
	compare(x, [0, 1, 1, 2, 2, 3, 3, 4, 4])


if __name__ == '__main__':
	test()
